//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                  SonicR Control Panel.mq4                                 | 
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2013 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview, _v3:

This SonicR Control Panel indicator has important changes from the SonicR Control Panel last
released on 05/01/2011.  A list of the changes can be found at the bottom of this overview.  
Please be informed that this SonicR Control Panel also differs from the SonicR Chart Panel 
which was last released 07-16-2011, in that the SonicR Clock Panel has been integrated into
this indicator.  Also, the time zone shifted Pivots feature and the market open vLines feature
are re-included, with new coding eliminating the need for the inputs "DST_Server_GMT_Offset" 
and "DST_Zone".  This SonicR Control Panel improves upon the sophisticated features of it's
predecessors and is easier to use.

Remarks On the "Preface" Inputs-
You only need to be concerned with this section if you have selected to display PivotsTz or 
vLines at a time when there is no live data feed from your broker server (weekends/holidays). 
Broker time is required by the code that features PivotsTz and vLines.  When data feed is live
the broker time is available.  When data feed is not live it is not available and manually
input correct information must be used to compensate, hence, the two inputs in this section.
Do the following to get the correct "Broker_GMT_Offset" value to input.  While data feed is
live, temporarily add "GMT" and "Broker" to the Clock if the display does not already show
"G" and "B" in the labels.  Compare the +/- hours time difference between GMT and Broker time.
For example, if GMT is 11pm and broker time is 2am, then the difference is "3" hours.  if GMT 
is 1am and broker time is 9pm, then the difference is "-4" hours.  Enter the time difference  
as the "Broker_GMT_Offset".  PivotsTz and vLines will now function properly without live data 
by setting "Apply_it_now" to "true".  When the data feed is live the indicator automatically 
adjusts for broker server DST changes when they actually occur twice yearly.  You must check 
your "Broker_GMT_Offset" input twice yearly when there is a DST changeover on the broker server, 
and correct the input if the value has changed.  The default is set to "0" for a broker server 
(broker time) that is at GMT all year.  At GMT there are no DST changeovers. If your broker is
at GMT all year then this setting will never require change.

Preview of the Displayed Panel and Other Indicator Features-
This indicator places a colored panel in the upper left corner of the chart.  It consists of
three sections-
   1. A market section containing: 
      (1) The pair symbol and time frame. 
      (2) The spread. 
      (3) Ranges: the daily average range, and the range achieved for the day, 
      (4) Swaps: the long and the short swap rates, 
      (5) The time to the next candle. 
      (6) The market bid price.  This indicator can be used for both forex and non-forex 
          items. In the forex mode, you have the option to have the final digit in the price 
          appears smaller than the full size digits.  In the non-forex mode, all digits in the 
          price are full size.  In both modes, the price changes color as the full size digits 
          number increments or decrements.  
   2. The "SonicR" logo.
   3. The Time Zone Clock, which can be displayed or not displayed.
   
In addition to the three panel sections, this multi-function indicator provides the following
services, which are listed here, and more fully described below: 
   1. A special bid line.
   2. A background of level lines dividing the distance between whole numbers into quarters. 
   3. Daily or Fibonacci pivots, including mid-pivot lines.
   4. Average Range H/L lines for both the day and week.
   5. Day Separators. 
   6. Vertical lines at the Asian Open, London Open, New York Open, and London Close. 
   7. A time zone clock that displays seven time zones of your selection.

1. Special Bid Line-
You can select to place your own Bid Ray Line on the chart, which you can vary in thickness 
and color.  The Bid Ray Line ends just beyond the current candle.  It terminates with a 
Bid Dot that can be sized and that changes color(example: grn/red) as the bid price changes
(up/dn).  To display only your own Bid Ray Line, right click on the chart and select 
Chart Properties/Colors.  Make the Grid color = "None", otherwise the MT4 bid line is going
to be beneath your own, and will be visible from the end of your line to the chart edge.

The hard code values that determine where the Bid Line ends and where the Bid Dot is placed
are different for each chart zoom setting (1-6).  The chart zoom setting can be provided by 
the SonicR Filled Dragon.  If not on the chart list of indicators, the SonicR PVA Candles 
indicator can provide the setting.  One or the other of these two indicators must be on the 
chart list of indicators in order to provide the chart zoom setting (1-6) needed by this
special Bid Line code.  Whichever indicator is used, it does not have to be turned on, but 
it's chart zoom setting (1-6) must be appropriate for the zoom that the chart is set to.  So, 
even if the indicator is turned off, when you change the zoom of the chart you must change the 
chart zoom setting of that indicator.  If both indicators are on the chart list of indicators, 
but are set to different chart zoom settings, the last indicator on the list of indicators is 
the controlling indicator for the special Bid line code.

2. Background Level Lines-
This indicator places level lines on your chart at key levels between whole numbers (00, 25,
50, 75 and 00 again).  The number of "sets" of these Whole, Half, and Quarter lines is fixed
by hard code for each TF chart.  The lines can individually be of any available color, style 
and width.  With Levels "On", the half and quarter levels can be turned On/Off individually.   

3. Daily and Fibonacci Pivots -
   A. Main Pivot Line
      You can select to display the main pivot (PVT) for the day.  It can be constructed using 
      two lines.  One can be a wide background line of a highlighting color (Example: Yellow) 
      and the other can be a narrow top line of a darker color (Example: Gray).
   B. Support & Resistance Lines-
      You can select to display the SR lines using either the Daily or Fibonacci formulas.  
      The Fibonacci displays five levels.  Normally, the Daily displays only three levels, but 
      is extended to include five levels.  
   C. Broker or Time Zone Shifted Pivots
      You can display your broker server pivots, or you can select to display pivots based on  
      a different time zone.  New coding fully automates the complexities of PivotsTz, such 
      as timing the hour shift for DST twice yearly, depending on when your broker does that.
      Your choice to use PivotsTz requires only two simple inputs-
      I.  "PivotsTz_On" 
          Set this equal to "true".
      II. "PivotsTz_GMT_Offset" 
          This is your choice of time zone location relative to GMT in +/- hours (examples: 
          for GMT use "0", for one time zone east of GMT use "1").  This is a geographical 
          zone described by a fixed offset from GMT, regardless of DST consideratons. 
          Understand that by choosing PivotsTz when the next PivotsTz day occurs during your 
          broker server day, the PivotsTz lines will suddenly change. 
          
          Broker pivots display "^" in the label.  PivotsTz, other than the default setting,
          will display "*" in the label.  By default, PivotsTz are set to the time zone most 
          used for pivots in the forex market (GMT+3) and have nothing added to the label.      

4. Range High/Low Lines for the Day and Week-
You can display two horizontal lines, one for the computed range high target, and one for 
the computed range low target. The range is based on the averaging period you select 
(defaults: day=15, week=13).  The special range averaging process for the Day range lines
skips any brief Sunday sessions of some brokers, which would drag the average down.  The 
Week range lines use the standard ATR formula since brief Sunday sessions do not much affect
the results.  
   A. Two conditions determine where range lines appear....
      1. Condition #1 - the Day/Week range has not exceeded the computed average range. 
         * The RDH/RWH line is the computed average range distance above the session Low.
         * The RDL/RWL line is the computed average range distance below the session high.  
         * The lines will move as new highs/lows are achieved during the session.  
         * This display shows how far price can move in either direction before exceeding
           the computed average range.  
      2. Condition #2 - the Day/Week range has exceeded the computed average range. 
         * If price swings during the day/week TF cause the difference between the high
           and the low to equal the computed range, the range lines lock into place.
         * This display will clearly show any subsequent breakout of the range.        
   B. The purpose of the range lines is to provide a perspective on how far PA might move 
      during the trading day or week.  This can be of some guidance in selecting where to 
      exit a trade.  A swing trader might pay more attention to the RWH/RWL lines, and hold
      a trade into oncoming days.  However, a day trader looking for quick profits and 
      limited exposure to market price swings might pay more attention to the RDH/RDL lines.

5. Day Separators-
You can apply day separators to your chart with this option.  It lets you select color 
and line styles.  You can elect to show separators just for today and/or for any number 
of previous days.

6. Vertical Time Lines-
You can select to display vertical time lines for the London Open, New York Open, and the
London Close.  New coding in this release properly places these vLines without any special
inputs required from the user because it determines all year around the GMT offsets of the 
broker server, London and NYC (EST).  And the coding takes into account any changes at these
locations due to DST/nonDST changeovers when they are implemented, even if at different times 
at each location.  This is the information required to always have the vLines displayed on your
broker server chart at the places representing the proper local times in both London and NYC.

7. Time Zone Clock-
You can select to display seven time zones on the Clock Panel from a complete list of time 
zones provided.  If any of the time zones being displayed happens to coincide with the broker
server time, your local time, or GMT, then extra labels will automatically appear designating:
(B), (L), (G), (B,L), (B,G), (L,G) and (B,L,G).  With these labels you do not have to waste 
lines to also display the Broker, Local or GMT time zones. Near the top of the External 
Inputs is the control input for turning on/off the display of the Clock in the Panel.
   
8. Addtitional Remarks On Displaying Horizontal Lines and Labels-
The horizontal line groups can be displayed differently, one from another, using display 
numbers "1" and "2" in the section "Horizontal Lines Position Settings:"  If the selection 
number is outside the 1-2 range it will default to "2".  The horizontal line groups are: Levels,
PVT (central pivot), S&R pivots, Day Range H/L lines, and Week Range H/L lines. 
  
   "1" - Fullscreen: draws the lines across the chart and positions labels mid-chart.  
   "2" - Session: draws the lines only thru the current session and positions labels at 
         the start of current session until that moves off-chart.  Then labels are
         positioned mid-chart. 

List of Changes From Last Release (07/16/2011) of the SonicR Chart Panel:
01 - Added back the time zone shifted pivots, this time including mid-pivots and fibonacci
     pivots.  New coding automatically determines broker server time shifts, replaces 
     the previous DST coding, and eliminates the need to input a "DST_Zone".
02 - Added back the vlines for market open times (Lc, Ny, Lc).  New coding automatically 
     determines their proper placement all year around regardless of DST considerations.
     Included is the ability to choose whether or not to display the vLine labels.
03 - Fixed #3 line display choice "Current session to right" so that it now works on all 
     TFs, not just those below H4.
04 - Added Week range lines with Max/Min TF to show, minor label placement adjustment for 
     slightly wide week range labels, and reordered codes for Week range, Day range, and
     Pivots to construct in that order.
05 - Fixed code for the display of ranges lines so now the Day range lines are correct on 
     Weekly and Monthly charts, and the Week range lines are correct on Monthly charts.
06 - Added subordination control of horizontal lines/labels (Levels always subordinated).     
07 - Fixed levels to properly display for additional symbols.      
08 - Automated the adjustment of "Display_4_Shift_Line_Right" when changing chart zoom, 
     provided choice of five shift right positions that can be used for indenting the 
     different catagories of horizontal lines/labels when using the "4" display choice, and
     included automatic indent shifting left of range lines for when pivot S&R lines are 
     turned on while range lines are set to same shift right position.          
09 - Housekeeping changes to levels and pivots labels, including making central pivot label 
     the same for Daily & Fib formula, removal of MaxRight override label positioning, and
     faster execution of labeling code in general.
10 - Recoded display of symbol in panel to strip broker added characters.
11 - Incorporated the separate SonicR Clock Panel indicator into this indicator with new
     coding automatically highlighting time zones matching your Broker and Local time.
12 - Revised the External Inputs accordingly.     

Changes from previous release 04-04-2012 to current release 05-01-2012:
01 - Revised the code using imported time zone information to get broker, London, and NYC
     offsets from GMT (used for PivotsTz and vLines placement) to fix errant vLines at
     transitions between AM and PM hours. 
02 - Revised code for current session vLines to preclude stacking of these lines at the
     beginning of "tail end" Sunday sessions that some brokers have.
03 - Improved coding of daily range value.        

Changes from previous release 05-01-2012 to current release 05-25-2013:
01 - Changes to Panel.....
     * Added option to display or not display the Clock.
     * Added Swap Rates.
     * Enhanced the Bid price display and simplified coding to remove the necessity to 
       choose between forex 5-digit and outdated 4-digit modes.
     * Improved coding so Panel and Clock will always be on top of drawn chart lines.       
02 - Improvements to Bid Line display.....
     * Corrected error in bid line draw on lowest zoom setting and revised Bid line/dot 
       termination to be based on chart zoom setting, which is derived from either the
       SonicR Filled Dragon indicator or the SonicR PVA Candles indicator.
05 - Changes to Pivots display.....
     * Added ability to limit the number of SR levels drawn.      
06 - Substantial rework/simplification of horizontal lines features and label coding.....
     * Removed test for "Display_Number" of horizontal lines at startup.
     * Removed all but two line positioning options.
     * Removed labels for Levels, the Line Sets Override feature, and the option to
       position differently the individual Levels catagories.
     * Removed feature to display prices in line labels.
     * Moved Session display of labels one position to the right.
06 - Additions and improvements to range lines.....
     * Simplified basic coding.       
     * Revised Day and Week Range line coding so that if chart TF is >240 with the display 
       is set to "0" or "1", then display "2" is used with "0" shift so the lines are 
       drawn from current candle to chart right. 
     * Added "Min TF" for showing Ranges for Day.  
     * Revised Range Week label coding so even if set to display, it can be set to not 
       display on lower TFs if the Range Day labels are also set to not display.
     * Removed unneeded specialized label coding for wide Week labels.
07 - Additions to vLines.....
     * Corrected code for Separators & vLines (to the minute exactness).
     * Added Max_TF to show vLines of prior days so their display can terminate on a 
       different TF than for the current day vLines.
     * Added option to show "Ao" (Asian Open).                
08 - Housekeeping changes for ease of use or enhanced speed.....
     * Re-structured External Inputs with phraseology changes.
     * Removed voided coding artifacts from previous releases.
     * Employed "nesting" coding for conditions testing.
     * Eliminated processing time zone information not needed for "active" services. 

                                                                  - Traderathome, 05-25-2013                                                                  
----------------------------------------------------------------------------------------------
Acknowlegements:
CaveMan - who generously devoted his time and programming skills to produce the PivotsTz time 
          zone shift coding, designed to handle any situation involving gaps between trading 
          days and/or partial trading days.         
Kent    - for his "pips4life" P4L clock code to import and use time zone information. 

----------------------------------------------------------------------------------------------
Suggested Settings:          White Chart              Black Chart 

Panel Section-        
Panel_Background_Color       C'238,238,242            C'25,25,25'
Symbol_And_TF_Color          Black                    C'208,208,208'
Spread_Color                 C'80,80,80'              C'120,120,120'
Range_Color                  C'80,80,80'              C'120,120,120'
Swaps_Color                  C'80,80,80'              C'120,120,120'
Candle_Time_Color            C'80,80,80'              C'120,120,120'
Bid_UP_Color                 ForestGreen              C'46,188,46'
Bid_DN_Color                 Crimson                  C'234,0,0'
Bid_Forex_5th_Digit_Large    C'140,140,140'           C'100,100,100'
Bid_Forex_5th_Digit_Small    Black                    C'205,205,205'
sonicS                       C'44,44,255'             C'4,162,255'
sonicO                       C'111,111,255'           C'0,115,183'
SonicN                       C'170,170,255'           C'0,75,121' 
SonicI                       C'252,154,156'           C'138,0,0'
sonicC                       C'239,61,105'            C'176,0,0'
sonicR                       C'227,15,68'             C'221,0,0' 

Bid Line Section-
Bid_Ray_Color                Black                    C'100,100,100' 
Bid_Dot_Up_Color             C'0,244,62'              Lime
Bid_Dot_Dn_Color             C'255,43,48'             Red 

Horizontal Line Labels Section-
Line_Labels_Color            MidnightBlue             DarkGray    

Levels Section-
Whole_Number_Color           C'235,235,252'           C'41,33,73'         
Half_Number_Color            C'210,251,210'           C'0,53,0'              
Quarter_Number_Color         C'239,235,222'           C'43,33,19'  

Pivots Section-                              
DPV1_Back_Color              Yellow                   CLR_NONE   
DPV2_Top_Color               Silver                   C'230,216,0' 
R_Pivot_Color                C'255,53,17'             C'179,0,0'
S_Pivot_Color                C'64,64,255'             C'90,75,173' 
MidPivot_Color               Olive                    C'85,85,0' 

Range Lines Section-
RDH_Color                    C'255,90,62'             Red
RDL_Color                    C'106,106,255'           C'0,115,230'
RWH_Color                    C'255,90,62'             Red
RWL_Color                    C'106,106,255'           C'0,115,230' 

Separators Section-
Separators_Color             BlueViolet               C'85,85,85'

vLines Section-
vLines_Color                 Silver                   C'60,60,0' 
vLabels_Color                MidnightBlue             Olive  
                    
Clock Section-
Clock_MktClosed              C'80,80,80'              C'120,120,120'
Location_MktClosed           C'80,80,80'              C'120,120,120'
Clock_MktOpen                C'133,84,35'             C'174,111,47'  
Location_MktOpen             C'0,149,34'              C'36,157,36'                                                                                                                                                            
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

#include <stdlib.mqh>
//See function documentation at http://msdn.microsoft.com/en-us/library/ms725473(VS.85).aspx
#import "kernel32.dll"
void GetLocalTime(int& LocalTimeArray[]);
void GetSystemTime(int& systemTimeArray[]);
int  GetTimeZoneInformation(int& LocalTZInfoArray[]);
bool SystemTimeToTzSpecificLocalTime(int& targetTZinfoArray[], 
     int& systemTimeArray[], int& targetTimeArray[]);
#import

//Global External Inputs-----------------------------------------------------------------------
extern bool   Indicator_On                    = true;
extern string Preface_________________        = "Get Broker_GMT_Offset w/Clock &";
extern string Instruction                     = "live feed.  Apply when no live feed.";
extern int    Broker_GMT_Offset               = 0;  
extern bool   Apply_it_now                    = false;
extern string End_Preface_____________        = "";

extern string _                               = "";
extern string Part_1                          = "Indicator Master Controls:";
extern bool   Show_Clock_in_Panel             = true; 
extern bool   Bid_Line_On                     = true;
extern bool   Levels_On                       = true;
extern bool   __Show_Half_Lines               = true;
extern bool   __Show_Quarter_Lines            = true;
extern int    __Levels_Max_TF                 = 60;
extern bool   Pivots_PVT_On                   = false;
extern int    __PVT_Max_TF                    = 60;  
extern bool   Pivots_SR_On                    = false;
extern int    __Show_Levels_Thru_2345         = 5;
extern bool   __Show_MidPivots                = true; 
extern int    __SR_Max_TF                     = 60;  
extern bool   Pivots_Fibonacci                = false;
extern bool   PivotsTz_On                     = true;
extern int    __GMT_Offset                    = 3;
extern bool   Range_Day_On                    = true;
extern int    __Day_Min_TF                    = 1;
extern int    __Day_Max_TF                    = 240;
extern bool   Range_Week_On                   = true; 
extern int    __Week_Min_TF                   = 15; 
extern int    __Week_Max_TF                   = 240;
extern bool   Separators_On                   = false;
extern int    __Prior_Days_To_Show            = 5;
extern int    __Separators_Max_TF             = 60;
extern bool   vLines_On                       = true;
extern int    __Current_Session_Max_TF        = 60;
extern int    __Prior_Sessions_To_Show        = 0;
extern int    __Prior_Sessions_Max_TF         = 30;
extern bool   __Show_Session_Ao               = false;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";
 
extern string __                              = "";
extern string Part_2                          = "Panel Settings:";
extern bool   Use_For_Forex                   = true;
extern bool   __Use_Full_Size_Fractional_Pip  = true; 
extern int    Digits_To_Show_In_Spread        = 2;
extern int    Days_In_Range_Day_Avg           = 15; 
extern int    Weeks_In_Week_ATR               = 13;   
extern color  Panel_Background_Color          = C'238,238,242';
extern color  Symbol_And_TF_Color             = Black; 
extern color  Spread_Color                    = C'80,80,80'; 
extern color  Range_Color                     = C'80,80,80'; 
extern color  Swaps_Color                     = C'80,80,80'; 
extern color  Candle_Time_Color               = C'80,80,80'; 
extern color  Bid_UP_Color                    = ForestGreen;
extern color  Bid_DN_Color                    = Crimson;
extern color  Bid_Forex_5th_Digit_Large       = C'140,140,140'; 
extern color  Bid_Forex_5th_Digit_Small       = Black;  
extern color  sonicS                          = Blue;
extern color  sonicO                          = C'111,111,255';
extern color  sonicN                          = C'170,170,255';
extern color  sonicI                          = C'252,154,156';  
extern color  sonicC                          = C'239,61,105';    
extern color  sonicR                          = C'227,15,68';   

extern string ___                             = "";
extern string Part_3                          = "Bid Line Settings:";
extern color  Bid_Ray_Color                   = Black;
extern int    Bid_Ray_LineStyle_01234         = 0;  
extern int    Bid_Ray_Thickness               = 1;
extern color  Bid_Dot_Up_Color                = C'0,244,62';   
extern color  Bid_Dot_Dn_Color                = C'255,43,48';       
extern int    Bid_Dot_Size                    = 9;

extern string ____                            = "";
extern string Part_4                          = "Horizontal Lines Position Settings:";
extern string note_4_0                        = "Choose line display number.";
extern string note_4_1                        = "1 = Fullscreen";
extern string note_4_2                        = "2 = Session";
extern int    Levels_Display_Number           = 1;
extern int    Pivot_PVT_Display_Number        = 2;
extern int    Pivot_SR_Display_Number         = 2;
extern int    Range_Day_Display_Number        = 2;
extern int    Range_Week_Display_Number       = 2;
extern bool   Subordinate_Lines               = false;

extern string _____                           = "";
extern string Part_5                          = "Horizontal Lines Labels Settings:";
extern bool   Pivot_PVT_Label_On              = true;
extern bool   Pivot_SR_Labels_On              = true;
extern bool   Pivot_Mid_Labels_On             = true;
extern bool   Range_Day_Labels_On             = true;
extern bool   Range_Week_Labels_On            = true;
extern bool   __Off_If_Day_Labels_Off         = true;
extern color  Labels_Color                    = MidnightBlue;
extern string FontStyle                       = "Verdana";
extern int    FontSize                        = 7;
extern bool   Subordinate_Labels              = false;

extern string ______                          = "";
extern string Part_6                          = "Levels Settings:"; 
extern color  Level_Whole_Color               = C'235,235,252';   
extern int    Level_Whole_Style_01234         = 0;
extern int    Level_Whole_Width_12345         = 3;
extern color  Level_Half_Color                = C'210,251,210'; 
extern int    Level_Half_Style_01234          = 0;
extern int    Level_Half_Width_12345          = 2;
extern color  Level_Quarter_Color             = C'239,235,222';  
extern int    Level_Quarter_Style_01234       = 0;
extern int    Level_Quarter_Width_12345       = 1;

extern string _______                         = "";
extern string Part_7                          = "Pivots Settings:"; 
extern color  PVT1_Back_Color                 = Yellow;
extern int    PVT1_Style_01234                = 0;
extern int    PVT1_Width_12345                = 3; 
extern color  PVT2_Top_Color                  = Silver;
extern int    PVT2_Style_01234                = 0;
extern int    PVT2_Width_12345                = 1; 
extern color  R_Pivot_Color                   = C'255,53,17'; 
extern int    R_Style_01234                   = 2;
extern int    R_Width_12345                   = 1; 
extern color  S_Pivot_Color                   = C'64,64,255';
extern int    S_Style_01234                   = 2;
extern int    S_Width_12345                   = 1;
extern color  MidPivots_Color                 = Olive;
extern int    mP_Style_01234                  = 2;
extern int    mP_Width_12345                  = 1;

extern string ________                        = "";
extern string Part_8                          = "Range Day Settings:";
extern color  RDH_Color                       = C'255,90,62';
extern color  RDL_Color                       = C'106,106,255';
extern int    RD_Style_01234                  = 0;    
extern int    RD_Width_12345                  = 2;

extern string _________                       = "";
extern string Part_9                          = "Range Week Settings:";
extern color  RWH_Color                       = C'255,90,62';
extern color  RWL_Color                       = C'106,106,255';
extern int    RW_Style_01234                  = 0;    
extern int    RW_Width_12345                  = 3;

extern string __________                      = "";
extern string Part_10                         = "Separators Settings:";
extern color  Separators_Color                = Silver;  
extern int    Separators_Style_01234          = 2;    
extern int    Separators_Width_12345          = 1;
extern bool   Separators_Thru_SubWindows      = true;

extern string ___________                     = "";
extern string Part_11                         = "vLines & Labels Settings:";
extern color  vLines_Color                    = Silver; 
extern int    vLines_Style_01234              = 2;    
extern int    vLines_Width_12345              = 1;
extern bool   vLines_Thru_SubWindows          = false;
extern color  vLabels_Color                   = MidnightBlue;
extern bool   vLabels_On                      = false;
extern string vFontStyle                      = "Tahoma";
extern int    vFontSize                       = 7; 

extern string ____________                    = "";
extern string Part_12                         = "Clock Settings:";
extern color  Clock_MktClosed                 = C'80,80,80';  
extern color  Location_MktClosed              = C'80,80,80';        
extern color  Clock_MktOpen                   = C'133,84,35';   
extern color  Location_MktOpen                = C'0,149,34';     
extern bool   Show_AMPM_Time                  = false;
extern string Note_12_1                       = "Choosing Locations to Display:";
extern string Note_12_2                       = "Select 7 maximum from list,";
extern string Note_12_3                       = "0 = display off.";
extern string Note_12_4                       = "1 = Display on.";
extern int    Broker                          = 0;
extern int    Local                           = 0; //Your computer's time.
extern int    Auckland                        = 0;
extern int    Sydney                          = 1;
extern int    Tokyo                           = 1;
extern int    HongKong                        = 1;
extern int    Jakarta                         = 0;
extern int    India                           = 0;
extern int    Dubai                           = 0;
extern int    Moscow                          = 0;
extern int    Israel                          = 0;
extern int    Helsinki                        = 1;
extern int    Frankfurt                       = 1;
extern int 	  London                          = 1; //DST London=GMT+1, NonDST London=GMT
extern int    GMT                             = 0; //No change btwn DST/summer & NonDST/winter
extern int    Brazil                          = 0;
extern int    NewYork                         = 1;
extern int    Central                         = 0;
extern int    Mexico                          = 0;
extern int    Mountain                        = 0;
extern int    Pacific                         = 0;

//Buffers, Constants and Variables-------------------------------------------------------------
bool       Deinitialized;

//Deinit
int        bld,obj_total,k;
string     name;

//Panel
color      Static_Price_Color, Static_Bid_Color, Bid_Dot_Color, Static_Bid_Dot_Color;
double     Poin,New_Price,Old_Price,level,Spread,ARg,RangeAchieved,dRG,dRange,wRange;
int        modifier,Color,Factor,G,h,i, m,s,t,LP,LS,T1,T2; 
string     item1  = "z[CP Panel] Box 1";  
string     item2  = "z[CP Panel] Box 2"; 
string     item7  = "z[CP Text] ID&TF";
string     item8  = "z[CP Text] Spread"; 
string     item9  = "z[CP Text] Range"; 
string     item0  = "z[CP Text] Swaps"; 
string     item10 = "z[CP Text] Time"; 
string     item11 = "z[CP Text] Price1";
string     item20 = "z[CP Text] Price2";
string     item12 = "z[CP Text] Logo1 S"; 
string     item13 = "z[CP Text] Logo1 o"; 
string     item14 = "z[CP Text] Logo1 n";     
string     item15 = "z[CP Text] Logo1 i"; 
string     item16 = "z[CP Text] Logo1 c";
string     item17 = "z[CP Text] Logo1 R"; 
string     item18 = "[CP Draw] Bid Line";
string     item19 = "[CP Draw] Bid Dot";
string     ID,Price,C,tab,tab1,timeleft; 

//Global Variable Controlling Bid Line/Dot Spacing
double     Chart_Zoom; 
string     Zoom = "Zoom_Setting"; 

//Levels
color      linecolor;
double     linelevel, linestyle, linewidth, BL;
int        j,Line1_Color, Line2_Color, Line3_Color; 
int        u1=00, u2=50, u3=25, u4=75;        
int        v1, ssp, ssp1, NumberOfRanges;
string     linename;

//Pivots
double     p,d,q,r1,r2,r3,r4,r5,s1,s2,s3,s4,s5;   
double     today_high, today_low, yesterday_high, yesterday_open, 
           yesterday_low, yesterday_close; 
int        tzdiff, idxfirstbaryesterday, idxlastbaryesterday, idxfirstbartoday,
           idxbar, tzdiffsec, barsperday, dayofweektoday, dayofweektofind;                                       
string     prefix;

//Range Day
double     dRangeHigh,dRangeLow,HiToday,LoToday,dRH,dRL;
int        ii,iii,x,xx,r,Hbarshift,Lbarshift,rdc,rwc,RDdisplay, RDshift;

//Range Week
int        wHbarshift, wLbarshift, RWdisplay, RWshift;
datetime   Weekshift;
double     HiWeek, LoWeek;
double     wRangeAvg, wRangeHigh, wRangeLow, wRH, wRL;

//Horizontal Lines Placement
datetime   startline, stopline;
int        a,b,c,R2,T,T4;  
string     line, dt;

//Horizontal Labels Placement
datetime   startlabel; 
string     label, spc;

//vLines & Labels
bool       vLabels_Chart_Top      = true; 
datetime   shift, T3;
double     top, bottom, scale,YadjustTop,YadjustBot,vLabels_Dist_Above_Border,time2,time4;
int        bps, MF,time3;
int        vLabels_Dist_To_Border = 1;
string     vline,vlabel;
string     vLabels_Asian_Open     = "      Ao";
string     vLabels_London_Open    = "      Lo";
string     vLabels_NewYork_Open   = "      Ny";
string     vLabels_London_Close   = "      Lc";
string     Label_Day_Separator    = "day";       

//Get GMT Offsets in Support of PivotsTz and Market vLines                    
int        GMT_Ref_Time, London_GMT_Offset, NYC_GMT_Offset; 
int        GMT_At_Lo, GMT_At_NYo, GMT_At_Lc, GMT_At_Ao; 

//Import Time Zone Data in Support of Clock, PivotsTz and Market vLines
bool       FLAG_match, FLAG2_match, FLAG_static_broker;
datetime   Local_Time,BrokerDT,AucklandDT,SydneyDT,TokyoDT,ChinaDT,JakartaDT,IndiaDT,
           DubaiDT,MoscowDT,IsraelDT,HelsinkiDT,BerlinDT,LondonDT,UTC,BrazilDT,NewYorkDT,
           CentralDT,MexicoDT,MountainDT,PacificDT,StaticBrokerDT;
int        ClockFontSize, TimezoneFontSize, LineSpacing, z, time, place, offset;              
int        AucklandTZInfoArray[43],SydneyTZInfoArray[43],TokyoTZInfoArray[43],
           ChinaTZInfoArray[43],JakartaTZInfoArray[43],IndiaTZInfoArray[43],
           DubaiTZInfoArray[43],MoscowTZInfoArray[43],IsraelTZInfoArray[43],
           HelsinkiTZInfoArray[43],BerlinTZInfoArray[43],LondonTZInfoArray[43],
           BrazilTZInfoArray[43],NewYorkTZInfoArray[43],CentralTZInfoArray[43],
           MexicoTZInfoArray[43],MountainTZInfoArray[43],PacificTZInfoArray[43],
           LocalTZInfoArray[43];          
int        LocalTimeArray[4],systemTimeArray[4],AucklandTimeArray[4],SydneyTimeArray[4],
           TokyoTimeArray[4],ChinaTimeArray[4],JakartaTimeArray[4],IndiaTimeArray[4],
           DubaiTimeArray[4],MoscowTimeArray[4],IsraelTimeArray[4],HelsinkiTimeArray[4],
           BerlinTimeArray[4],LondonTimeArray[4],BrazilTimeArray[4],NewYorkTimeArray[4],
           CentralTimeArray[4],MexicoTimeArray[4],MountainTimeArray[4],PacificTimeArray[4];         
string     Locals,Brokers,Aucklands,Sydneys,Tokyos,Chinas,Jakartas,Indias,Dubais,
           Moscows,Israels,Helsinkis,Berlins,Londons,UTCs,Brazils,NewYorks,Centrals,
           Mexicos,Mountains,Pacifics;           
string     Brokerp, Localp, Aucklandp, Sydneyp, Tokyop, Chinap, Jakartap,  
           Indiap, Dubaip, Moscowp, Israelp, Helsinkip, Berlinp, Londonp, UTCp, 
           Brazilp, NYp, Centralp, Mexicop, Mountainp, Pacificp, FontName; 
string     timeStr,LocalTimeS,sMonth,sDay,sHour,sMin,sSec;
int        hour,nYear,nMonth,nDay,nHour,nMin,nSec,nMilliSec;  
                      
//Clock - Normal hrs 8-17 (9 hrs), Sydney 9-18 (9 hrs) and Tokyo 9-18 (9 hrs)   
int        LocalOpenHour        =  8;    
int        LocalCloseHour       = 17;
int        SydneyLocalOpenHour  =  9;       
int        SydneyLocalCloseHour = 18;
int        TokyoLocalOpenHour   =  9;        
int        TokyoLocalCloseHour  = 18; 
      
//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
  {
  //With the first DLL call below, the program will exit (and stop) automatically after one alert. 
  if(!IsDllsAllowed() ) 
    {
    Alert("SonicR Control Panel: Allow DLL Imports"); return(false);  
    }
  
  //Signal State of Non-deinitialization 
  Deinitialized = false;
  
  //Get Zoom value for Bid Line/Dot placement
  Chart_Zoom = GlobalVariableGet(Zoom);    
  
  //Panel - setting background width   
  G= 124; if(Show_AMPM_Time) {G= 142;}
             
  //Panel - determine display of Market Price  
  if(Digits == 2) {Use_For_Forex = false;}          
  if(Digits == 5 || Digits == 3) {Factor = 10;} 
  else {Factor = 1;} //cater for 5 digits 
    
  //Panel and Levels - Check for unconventional Point digits number. 
  //Used for display of market price and Levels.
  if (Point == 0.00001) {Poin = 0.0001;} //5 digits     
  else {if (Point == 0.001) {Poin = 0.01;} //3 digits
  else {Poin = Point;}} //Normal 4 digits    
  if (StringSubstr(Symbol(),0,6) == "XAUUSD") {Poin = 1;}   
  else {if (StringSubstr(Symbol(),0,6) == "USDMXN") {Poin = 0.001;}
  else {if (StringSubstr(Symbol(),0,6) == "USDCZK") {Poin = 0.001;} }} 
  
  //Panel - Make Labels
  MakeLabel( item1, 0, 0);
  if(Show_Clock_in_Panel) {MakeLabel( item2, G, 0);}
  MakeLabel( item7, 0,2);  
  MakeLabel( item8, 27,19);
  MakeLabel( item9, 27,29);  
  MakeLabel( item0, 27,39);
  MakeLabel( item10, 27,49);     
  MakeLabel( item11, 7,60);
  if(Use_For_Forex) 
    {
    if(__Use_Full_Size_Fractional_Pip) {MakeLabel( item20, 7,60);}
    else {MakeLabel( item20, 7,61);} 
    }   
  MakeLabel( item12, 147,2);      
  MakeLabel( item13, 148,15); 
  MakeLabel( item14, 148,26); 
  MakeLabel( item15, 149,40);
  MakeLabel( item16, 148,50); 
  MakeLabel( item17, 147,64);  
       
  //Levels - set ranges for chart TFs 
        if(Period() == 1) {NumberOfRanges = 4;}
  else {if(Period() == 5) {NumberOfRanges = 7;}
  else {if(Period() == 15) {NumberOfRanges = 15;}
  else {if(Period() == 30) {NumberOfRanges = 15;}   
  else {if(Period() == 60) {NumberOfRanges = 20;}  
  else {if(Period() == 240) {NumberOfRanges = 30;}  
  else {if(Period() == 1440) {NumberOfRanges = 30;} 
  else {if(Period() >= 10080) {NumberOfRanges = -1;} }}}}}}}

  //Import Time Zone Data  required for PivotsTZ and vLines      
  StaticBrokerDT = TimeCurrent();           
  Get03TimeZoneInfo(SydneyTZInfoArray, LondonTZInfoArray, NewYorkTZInfoArray); 
   
  //Clock - Import rest of Time Zone Data required, and make Clock Labels   
  if(Show_Clock_in_Panel) 
    {   
    Get15TimeZoneInfo(AucklandTZInfoArray, TokyoTZInfoArray, 
    ChinaTZInfoArray, JakartaTZInfoArray, IndiaTZInfoArray, DubaiTZInfoArray, 
    MoscowTZInfoArray, IsraelTZInfoArray, HelsinkiTZInfoArray, BerlinTZInfoArray, 
    BrazilTZInfoArray, CentralTZInfoArray, 
    MexicoTZInfoArray, MountainTZInfoArray, PacificTZInfoArray);      
        
    FLAG_match  = false; 
    FLAG2_match = false; 
    LineSpacing = 11;//12 
    top   = 3;   //vertical start text
    offset= 0;   //vertical "LineSpacing" additions to "top" per line of text
    time  = 180; //horizontal start time (same for 24hr and am/pm times), 140
    place = 215; //horizontal start location (if using 24hr format), 177
    if(Show_AMPM_Time) //shift to right to accomodate extra time characters
      {                
      place= 232; //shift location right, 196
      }   
  	 //Begin world timezones, listed IN ORDER!
    if(Broker == 1)    	  	
      {
      MakeLabel( "z[CP Time] Brokert", time, top+offset );
      MakeLabel( "z[CP Time] Brokerl", place, top+offset );
     	offset+=LineSpacing;
      } 
    if(Auckland == 1)         	
      {                  	     	       	        
   	MakeLabel( "z[CP Time] Aucklandt", time, top+offset );         
   	MakeLabel( "z[CP Time] Aucklandl", place, top+offset );
   	offset+=LineSpacing;    
      }
    if(Sydney == 1)      
      {               
   	MakeLabel( "z[CP Time] Sydneyt", time, top+offset );
   	MakeLabel( "z[CP Time] Sydneyl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Tokyo == 1)
      {
   	MakeLabel( "z[CP Time] Tokyot", time, top+offset );
   	MakeLabel( "z[CP Time] Tokyol", place, top+offset );
   	offset+=LineSpacing;
      }
    if(HongKong == 1)       
      {
   	MakeLabel( "z[CP Time] Chinat", time, top+offset );
   	MakeLabel( "z[CP Time] Chinal", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Jakarta == 1)        
      {
   	MakeLabel( "z[CP Time] Jakartat", time, top+offset );
   	MakeLabel( "z[CP Time] Jakartal", place, top+offset );
   	offset+=LineSpacing;
      }
    if(India == 1)      
      {
   	MakeLabel( "z[CP Time] Indiat", time, top+offset );
   	MakeLabel( "z[CP Time] Indial", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Dubai == 1)        
      {
   	MakeLabel( "z[CP Time] Dubait", time, top+offset );
   	MakeLabel( "z[CP Time] Dubail", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Moscow == 1)       
      {
   	MakeLabel( "z[CP Time] Moscowt", time, top+offset );
   	MakeLabel( "z[CP Time] Moscowl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Israel == 1)       
      {
   	MakeLabel( "z[CP Time] Israelt", time, top+offset );
   	MakeLabel( "z[CP Time] Israell", place, top+offset );
   	offset+=LineSpacing;
      }      
    if(Helsinki == 1)        
      {                           	       
   	MakeLabel( "z[CP Time] Helsinkit", time, top+offset );
   	MakeLabel( "z[CP Time] Helsinkil", place, top+offset );
   	offset+=LineSpacing; 	
      }      
    if(Frankfurt == 1)        
      {
   	MakeLabel( "z[CP Time] Berlint", time, top+offset );
   	MakeLabel( "z[CP Time] Berlinl", place, top+offset );
   	offset+=LineSpacing;
      }      
    if(London == 1)
      {         
	   MakeLabel( "z[CP Time] Londont", time, top+offset );
   	MakeLabel( "z[CP Time] Londonl", place, top+offset );
   	offset+=LineSpacing;
      }      
    if(GMT == 1)        
      {
   	MakeLabel( "z[CP Time] utct", time, top+offset );
   	MakeLabel( "z[CP Time] utcl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Brazil == 1)      
      {
   	MakeLabel( "z[CP Time] Brazilt", time, top+offset );
   	MakeLabel( "z[CP Time] Brazill", place, top+offset );
   	offset+=LineSpacing;
      }
    if(NewYork == 1)       
      { 
	   MakeLabel( "z[CP Time] NewYorkt", time, top+offset );
   	MakeLabel( "z[CP Time] NewYorkl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Central == 1)        
      {
   	MakeLabel( "z[CP Time] Centralt", time, top+offset );
   	MakeLabel( "z[CP Time] Centrall", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Mexico == 1)        
      { 
   	MakeLabel( "z[CP Time] Mexicot", time, top+offset );
   	MakeLabel( "z[CP Time] Mexicol", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Mountain == 1)        
      {
   	MakeLabel( "z[CP Time] Mountaint", time, top+offset );
   	MakeLabel( "z[CP Time] Mountainl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Pacific == 1)        
      {
	   MakeLabel( "z[CP Time] Pacifict", time, top+offset );
   	MakeLabel( "z[CP Time] Pacificl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Local == 1)        
      {     
      MakeLabel( "z[CP Time] Localt", time, top+offset );
      MakeLabel( "z[CP Time] Locall", place, top+offset );
      offset+=LineSpacing;
      }
    }                            

  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+       
int deinit()
  {   
  obj_total= ObjectsTotal();  
  for(k= obj_total; k>=0; k--)
    {
    name= ObjectName(k);
    if (StringSubstr(name,0,3)=="[CP"){ObjectDelete(name);}
    if (StringSubstr(name,0,4)=="z[CP"){ObjectDelete(name);}
    }
  //Comment ("");
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+         
int start()
  {
  //If Indicator is "Off" deinitialize only once, not every tick-------------------------------  
  if(!Indicator_On) 
    {
    if (!Deinitialized) {deinit(); Deinitialized = true;}
    return(0);
    }
            
  //If indicator is "ON", show deinit() is false and run---------------------------------------
  //deinit only [CP Draw} objects, deinit() remains false, and run
  Deinitialized = false;
  obj_total= ObjectsTotal();  
  for(k= obj_total; k>=0; k--)
    {
    name= ObjectName(k); 
    if (StringSubstr(name,0,9)=="[CP Draw]"){ObjectDelete(name);} 
    }

  //Up/down color selection when bid price changes--------------------------------------------- 
  //"New_Price" is the full-digits minus the last digit for forex application and is the
  //full-digits for non-forex applications, and controls when the Bid changes color.  
  //"Price" is the full-digits in all applications, and controls the Bid price displayed. 
  if(Use_For_Forex) 
    {
    New_Price = MathFloor(Bid/Poin)*Poin;  
    }         
  else 
    {
    New_Price = Bid;
    }
  Price=DoubleToStr(Bid, Digits);                     
  if(New_Price > Old_Price) 
    {
    Color = Bid_UP_Color;
    Static_Price_Color = Color;     
    Bid_Dot_Color = Bid_Dot_Up_Color;       
    Static_Bid_Dot_Color = Bid_Dot_Up_Color;   
    }
  else {if (New_Price < Old_Price) 
    {      
    Color = Bid_DN_Color; 
    Static_Price_Color = Color;  
    Bid_Dot_Color = Bid_Dot_Dn_Color;  
    Static_Bid_Dot_Color = Bid_Dot_Dn_Color;  
    }   
  else //if (New_Price == Old_Price)  
    {
    Color = Static_Price_Color;       
    Bid_Dot_Color = Static_Bid_Dot_Color;
    } }
  Old_Price = New_Price;
      
  //Panel--------------------------------------------------------------------------------------    
  //Background boxes      
  ObjectSet(item1, OBJPROP_BACK, false); 
  ObjectSetText(item1, "gg", 64, "Webdings", Panel_Background_Color);         
  //Clock Panel 
  if( Show_Clock_in_Panel)
    {       
    ObjectSet(item2, OBJPROP_BACK, false); 
    ObjectSetText(item2, "gg", 64, "Webdings", Panel_Background_Color);  
    }
     
  //Symbol and Time Frame
  ID = StringSubstr(Symbol(), 0, 6);        	 	         
  C = " "; if (Period()== 1)     C =C +  "    "+ID+"   M1";
     else {if (Period()== 5)     C =C +  "    "+ID+"   M5";
     else {if (Period()== 15)    C =C +  "   " +ID+"   M15";
     else {if (Period()== 30)    C =C +  "   " +ID+"   M30";
     else {if (Period()== 60)    C =C +  "    "+ID+"   H1";
     else {if (Period()== 240)   C =C +  "    "+ID+"   H4";
     else {if (Period()== 1440)  C =C +  "    "+ID+"   Day";
     else {if (Period()== 10080) C =C +  "   " +ID+"   Week";
     else {if (Period()== 43200) C =C +  "  "  +ID+"   Month"; }}}}}}}} 
  ObjectSetText(item7, C, 11, "Arial Bold", Symbol_And_TF_Color); 
                          
  //Spread
  Spread = MarketInfo(Symbol(), MODE_SPREAD);
  Spread = Spread/Factor;                                        
  ObjectSetText(item8, "Spread   "+
     DoubleToStr(Spread,Digits_To_Show_In_Spread), 7, "Arial", Spread_Color); 
   
  //Show Range data on one line     
  Ranges(Days_In_Range_Day_Avg);
  dRange = ARg/Days_In_Range_Day_Avg; 
  dRG    = (dRange/Point)/Factor; 
  HiToday    = iHigh(NULL,PERIOD_D1,0);
  LoToday	 = iLow (NULL,PERIOD_D1,0);	
  Hbarshift  = iHighest(NULL,0,MODE_HIGH,1440/Period(),0); 
  Lbarshift  = iLowest(NULL,0,MODE_LOW,1440/Period(),0);     
  RangeAchieved = ((HiToday - LoToday)/Point)/Factor;   
  ObjectSetText(item9, "Range    "+DoubleToStr(dRG,0)
    +", "+DoubleToStr(RangeAchieved,0), 7, "Arial", Range_Color);  

  //Show Swaps L/S on next line
  ObjectSetText(item0, "Swaps    "+DoubleToStr(MarketInfo(Symbol(),MODE_SWAPLONG),2)+
    ", "+DoubleToStr(MarketInfo(Symbol(),MODE_SWAPSHORT),2),7, "Arial", Swaps_Color);     
           
  //Show Candle Time on next line 	   
  if(Period()>1440) {timeleft = " (OFF)";}
  else {timeleft =TimeToStr(Time[0]+Period()*60-TimeCurrent(),TIME_MINUTES|TIME_SECONDS);}                  
  ObjectSetText(item10, "Candle   "+timeleft, 7, "Arial", Candle_Time_Color);          
  
  //Market Price 
  LP = StringLen(Price); 
  if(Digits != 3)    
    {                
	 tab= "    ";	 
	 if(LP<8) {tab=tab + " ";}
	 if(LP==5) {tab=tab + "  ";}
	 }      
  else 
    {
    tab= "     ";
    if(LP==6) {tab=tab + " ";} 	  		   	 	     	        
    }  
  
  //Show final Price digit as subordinate          
  if(Use_For_Forex)  
    {
    if(__Use_Full_Size_Fractional_Pip)
      {
      ObjectSetText(item11, tab+Price,14, "Arial Bold",Bid_Forex_5th_Digit_Large);  
      ObjectSetText(item20, tab+StringSubstr(Price,0,LP-1), 14, "Arial Bold",Color); 
      }
    else
      {   
      ObjectSetText(item11, tab+StringSubstr(Price,0,LP-1), 14, "Arial Bold",Color);
      tab1="";
            if(LP==6) {for(i=0; i<= 2.4*LP-1; i++) {tab1= tab1 + " ";} }
      else {if(LP==7) {for(i=0; i<= 2.5*LP-1; i++) {tab1= tab1 + " ";} }
      else {if(LP==8) {for(i=0; i<= 2.4*LP-1; i++) {tab1= tab1 + " ";} }  }}  
      ObjectSetText(item20, tab+tab1+StringSubstr(Price,LP-1,1),10, "Arial Bold",Bid_Forex_5th_Digit_Small);         
      }  
    }
        
  //Show all Price digits uniformly          
  else 
    { 
    ObjectSetText(item11, tab+Price, 14, "Arial Bold",Color);
    }         
  
  //Create SonicR Logo            
  ObjectSetText(item12, "S", 11, "Arial Bold",sonicS);      
  ObjectSetText(item13, "o", 10, "Arial Bold",sonicO);      
  ObjectSetText(item14, "n", 10, "Arial Bold",sonicN);   
  ObjectSetText(item15, "i", 10, "Arial Bold",sonicI);           
  ObjectSetText(item16, "c", 10, "Arial Bold",sonicC);       
  ObjectSetText(item17, "R", 11, "Arial Bold",sonicR); 
  
  //Clock, PivotsTZ and vLines-----------------------------------------------------------------
  //Go to subroutine to get TZ info needed for Clock, PivotsTZ and vLines
  if(Show_Clock_in_Panel || PivotsTz_On || vLines_On)
    {
    GetSystemTime(systemTimeArray);
    UTC = TimeArrayToTime(systemTimeArray); 
    }      
  if(Show_Clock_in_Panel || PivotsTz_On)
    {             
    BrokerDT = TimeCurrent();
    }   
  if(Show_Clock_in_Panel || vLines_On)
    {
    SystemTimeToTzSpecificLocalTime(SydneyTZInfoArray, systemTimeArray, SydneyTimeArray);
    SydneyDT   = TimeArrayToTime(SydneyTimeArray);      
    SystemTimeToTzSpecificLocalTime(LondonTZInfoArray, systemTimeArray, LondonTimeArray);
    LondonDT   = TimeArrayToTime(LondonTimeArray);   
    SystemTimeToTzSpecificLocalTime(NewYorkTZInfoArray, systemTimeArray, NewYorkTimeArray);
    NewYorkDT  = TimeArrayToTime(NewYorkTimeArray);
    }
    
  //PivotsTZ and vLines------------------------------------------------------------------------    
  //With required time zone data imported, define parameters for PivotsTZ and vLines  
  //if vLines is "on", then calculate GMT at Lo, Ny and Lc, and   
  if(vLines_On && Period()<= __Current_Session_Max_TF)
    {
    //Make 8am the "reference time" at GMT for London Open
    GMT_Ref_Time = 8;
     
    //Find GMT At Ao (when Sydney is at 9am local)
    int Sydney_GMT_Offset = (SydneyDT/3600 - UTC/3600);
    GMT_At_Ao = GMT_Ref_Time-Sydney_GMT_Offset+1; //+1 because Syndey opens 9am local, not 8am
         
    //Find GMT At Lo (when London is at 8am local)             
    London_GMT_Offset = LondonDT/3600 - UTC/3600;      
    GMT_At_Lo = GMT_Ref_Time-London_GMT_Offset;
                         
    //Find GMT At Ny (when New York is at 8am local)
    NYC_GMT_Offset = NewYorkDT/3600 - UTC/3600;
    GMT_At_NYo = GMT_Ref_Time-NYC_GMT_Offset; 
                           
    //Find GMT At Lc (nine hours after London Open)
    GMT_At_Lc = GMT_At_Lo + 9;
    }
          
  //if vLines is "on", or if PivotsTz is "on", then calculate "Broker_GMT_Offset".    
  if((vLines_On && Period()<= __Current_Session_Max_TF) ||
    (PivotsTz_On && Period()<= __PVT_Max_TF ))
    {
    //Compute "Broker_GMT_Offset" only when the data feed is live.  When the data feed 
    //is not live, then set "Apply_it_now" to "true" so that the manually input value
    //for "Broker_GMT_Offset" is used instead of computing it. 
    if(!Apply_it_now) 
      {
      Broker_GMT_Offset = BrokerDT/3600 - UTC/3600;
      }                  
    }      
    
  //Clock--------------------------------------------------------------------------------------    
  //Go to subroutine for rest of TZ info needed for Clock
  if(Show_Clock_in_Panel) 
    {     
    SystemTimeToTzSpecificLocalTime(AucklandTZInfoArray, systemTimeArray, AucklandTimeArray);
    AucklandDT = TimeArrayToTime(AucklandTimeArray);      
    //Sydney  
    SystemTimeToTzSpecificLocalTime(TokyoTZInfoArray, systemTimeArray, TokyoTimeArray);
    TokyoDT    = TimeArrayToTime(TokyoTimeArray);      
    SystemTimeToTzSpecificLocalTime(ChinaTZInfoArray, systemTimeArray, ChinaTimeArray);
    ChinaDT    = TimeArrayToTime(ChinaTimeArray);      
    SystemTimeToTzSpecificLocalTime(JakartaTZInfoArray, systemTimeArray, JakartaTimeArray);
    JakartaDT  = TimeArrayToTime(JakartaTimeArray);      
    SystemTimeToTzSpecificLocalTime(IndiaTZInfoArray, systemTimeArray, IndiaTimeArray);
    IndiaDT    = TimeArrayToTime(IndiaTimeArray);      
    SystemTimeToTzSpecificLocalTime(DubaiTZInfoArray, systemTimeArray, DubaiTimeArray);
    DubaiDT    = TimeArrayToTime(DubaiTimeArray);      
    SystemTimeToTzSpecificLocalTime(MoscowTZInfoArray, systemTimeArray, MoscowTimeArray);
    MoscowDT   = TimeArrayToTime(MoscowTimeArray);      
    SystemTimeToTzSpecificLocalTime(IsraelTZInfoArray, systemTimeArray, IsraelTimeArray);
    IsraelDT   = TimeArrayToTime(IsraelTimeArray);      
    SystemTimeToTzSpecificLocalTime(HelsinkiTZInfoArray, systemTimeArray, HelsinkiTimeArray);
    HelsinkiDT = TimeArrayToTime(HelsinkiTimeArray);        
    SystemTimeToTzSpecificLocalTime(BerlinTZInfoArray, systemTimeArray, BerlinTimeArray);
    BerlinDT   = TimeArrayToTime(BerlinTimeArray);      
    //London  
    SystemTimeToTzSpecificLocalTime(BrazilTZInfoArray, systemTimeArray, BrazilTimeArray);
    BrazilDT   = TimeArrayToTime(BrazilTimeArray);   
    //NewYork  
    SystemTimeToTzSpecificLocalTime(CentralTZInfoArray, systemTimeArray, CentralTimeArray);
    CentralDT  = TimeArrayToTime(CentralTimeArray);      
    SystemTimeToTzSpecificLocalTime(MexicoTZInfoArray, systemTimeArray, MexicoTimeArray);
    MexicoDT   = TimeArrayToTime(MexicoTimeArray);      
    SystemTimeToTzSpecificLocalTime(MountainTZInfoArray, systemTimeArray, MountainTimeArray);
    MountainDT = TimeArrayToTime(MountainTimeArray);      
    SystemTimeToTzSpecificLocalTime(PacificTZInfoArray, systemTimeArray, PacificTimeArray);
    PacificDT  = TimeArrayToTime(PacificTimeArray);      
    GetLocalTime(LocalTimeArray);
    Local_Time = TimeArrayToTime(LocalTimeArray);
      
    //Go to subroutine to convert time data to string format for use in clock labels
    Brokers   = Time_To_String( TimeCurrent() );   
    Aucklands = Time_To_String( AucklandDT  );
    Sydneys   = Time_To_String( SydneyDT  );
    Tokyos    = Time_To_String( TokyoDT  );
    Chinas    = Time_To_String( ChinaDT  );
    Jakartas  = Time_To_String( JakartaDT  );
    Indias    = Time_To_String( IndiaDT  );
    Dubais    = Time_To_String( DubaiDT  );
    Moscows   = Time_To_String( MoscowDT  );
    Israels   = Time_To_String( IsraelDT  );
    Helsinkis = Time_To_String( HelsinkiDT  );
    Berlins   = Time_To_String( BerlinDT  );
    Londons   = Time_To_String( LondonDT  );
    UTCs      = Time_To_String( UTC );
    Brazils   = Time_To_String( BrazilDT  );
    NewYorks  = Time_To_String( NewYorkDT  );
    Centrals  = Time_To_String( CentralDT  );
    Mexicos   = Time_To_String( MexicoDT  );
    Mountains = Time_To_String( MountainDT  );
    Pacifics  = Time_To_String( PacificDT  );
    Locals    = Time_To_String( Local_Time  ); 
    if(UTC <= BrokerDT+600 && UTC >= BrokerDT-600) {FLAG2_match = true;}   
            
    //Formating clock labels 
    FontName = "Arial";
    ClockFontSize = 7;
    TimezoneFontSize = 7;        
  
    //Populate Clock Labels           
    if(Broker == 1)  
      {
      if(!FLAG_match) {
      if(BrokerDT == Local_Time)  {Brokerp= "Broker  (L)";}                              
      else {Brokerp= "Broker";} 
      if(BrokerDT <= UTC + 600  && BrokerDT >= UTC - 600)
        {
        if(Brokerp== "Broker  (L)") 
          {
          Brokerp= StringSubstr(Brokerp,0,StringLen(Brokerp)-1) + ",GMT)";     
          } 
        else
          {
          Brokerp= StringSubstr(Brokerp,0) + "  (GMT)";
          } 
        }
      }       
	   ObjectSetText( "z[CP Time] Brokert", Brokers, ClockFontSize, FontName, Clock_MktClosed );
      ObjectSetText( "z[CP Time] Brokerl", Brokerp, TimezoneFontSize, FontName, Location_MktClosed );
      }
    if(Auckland == 1)                   
      {
      if(!FLAG_match) {
      if((AucklandDT <= BrokerDT+600 && AucklandDT >= BrokerDT-600)  &&
        (AucklandDT <= Local_Time+600 && AucklandDT >= Local_Time-600))
        {Aucklandp= "Auckland  (B,L)";}         
      else {if ((AucklandDT <= BrokerDT+600) && (AucklandDT >= BrokerDT-600)) 
        {Aucklandp= "Auckland  (B)";}
      else {if(AucklandDT == Local_Time) 
        {Aucklandp= "Auckland  (L)";}               
      else {Aucklandp= "Auckland";} }} }
   	if(TimeDayOfWeek(AucklandDT) != 0 && TimeDayOfWeek(AucklandDT) != 6 
   	  && TimeHour(AucklandDT) >= LocalOpenHour && TimeHour(AucklandDT) < LocalCloseHour)
        {
   	  ObjectSetText( "z[CP Time] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "z[CP Time] Aucklandl", Aucklandp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
      else
   	  {
   	  ObjectSetText( "z[CP Time] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Aucklandl", Aucklandp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Sydney == 1)        
      {
      if(!FLAG_match) {
      if((SydneyDT <= BrokerDT+600 && SydneyDT >= BrokerDT-600)  &&
        (SydneyDT <= Local_Time+600 && SydneyDT >= Local_Time-600))
        {Sydneyp= "Sydney  (B,L)";}       
      else {if ((SydneyDT <= BrokerDT+600) && (SydneyDT >= BrokerDT-600)) 
        {Sydneyp= "Sydney  (B)";}
      else {if(SydneyDT == Local_Time) 
        {Sydneyp= "Sydney  (L)";}               
      else {Sydneyp= "Sydney";} }} }     	     	      	     
   	if(TimeDayOfWeek(SydneyDT) != 0 && TimeDayOfWeek(SydneyDT) != 6 
   	  && TimeHour(SydneyDT) >= SydneyLocalOpenHour && TimeHour(SydneyDT) < SydneyLocalCloseHour)
   	  {
   	  ObjectSetText( "z[CP Time] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "z[CP Time] Sydneyl", Sydneyp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Sydneyl", Sydneyp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Tokyo == 1)           
      {
      if(!FLAG_match) {
      if((TokyoDT <= BrokerDT+600 && TokyoDT >= BrokerDT-600)  &&
        (TokyoDT <= Local_Time+600 && TokyoDT >= Local_Time-600))
        {Tokyop= "Tokyo  (B,L)";}          
      else {if ((TokyoDT <= BrokerDT+600) && (TokyoDT >= BrokerDT-600)) 
        {Tokyop= "Tokyo  (B)";}
      else {if(TokyoDT == Local_Time) 
        {Tokyop= "Tokyo  (L)";}                 
      else {Tokyop= "Tokyo";} }} }        	      	         	         
   	if(TimeDayOfWeek(TokyoDT) != 0 && TimeDayOfWeek(TokyoDT) != 6 
   	  && TimeHour(TokyoDT) >= TokyoLocalOpenHour && TimeHour(TokyoDT) < TokyoLocalCloseHour)
   	  {
   	  ObjectSetText( "z[CP Time] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "z[CP Time] Tokyol", Tokyop, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Tokyol", Tokyop, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(HongKong == 1)        
      {
      if(!FLAG_match) { 
      if((ChinaDT <= BrokerDT+600 && ChinaDT >= BrokerDT-600)  &&
        (ChinaDT <= Local_Time+600 && ChinaDT >= Local_Time-600))
        {Chinap= "HongKong  (B,L)";}          
      else {if ((ChinaDT <= BrokerDT+600) && (ChinaDT >= BrokerDT-600)) 
        {Chinap= "HongKong  (B)";}
      else {if(ChinaDT == Local_Time) 
        {Chinap= "HongKong  (L)";}                      
      else {Chinap= "HongKong";} }} }               	     	           
   	if(TimeDayOfWeek(ChinaDT) != 0 && TimeDayOfWeek(ChinaDT) != 6 
   	  && TimeHour(ChinaDT) >= LocalOpenHour && TimeHour(ChinaDT) < LocalCloseHour)
   	  {
    	  ObjectSetText( "z[CP Time] Chinat", Chinas, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "z[CP Time] Chinal", Chinap, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Chinat", Chinas, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Chinal", Chinap, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }      
    if(Jakarta == 1)   
      {
      if(!FLAG_match) { 
      if((JakartaDT <= BrokerDT+600 && JakartaDT >= BrokerDT-600)  &&
        (JakartaDT <= Local_Time+600 && JakartaDT >= Local_Time-600))
        {Jakartap= "Jakarta  (B,L)";}            
      else {if ((JakartaDT <= BrokerDT+600) && (JakartaDT >= BrokerDT-600)) 
        {Jakartap= "Jakarta  (B)";}
      else {if(JakartaDT == Local_Time) 
        {Jakartap= "Jakarta  (L)";}                   
      else {Jakartap= "Jakarta";} }} }            	     	         	     
   	if(TimeDayOfWeek(JakartaDT) != 0 && TimeDayOfWeek(JakartaDT) != 6
   	  && TimeHour(JakartaDT) >= LocalOpenHour && TimeHour(JakartaDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "z[CP Time] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "z[CP Time] Jakartal", Jakartap, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Jakartal", Jakartap, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(India == 1)      
      {
      if(!FLAG_match) {   
      if((IndiaDT <= BrokerDT+600 && IndiaDT >= BrokerDT-600)  &&
        (IndiaDT <= Local_Time+600 && IndiaDT >= Local_Time-600))
        {Indiap= "India  (B,L)";}           
      else {if ((IndiaDT <= BrokerDT+600) && (IndiaDT >= BrokerDT-600)) 
        {Indiap= "India  (B)";}
      else {if(IndiaDT == Local_Time) 
        {Indiap= "India  (L)";}                     
      else {Indiap= "India";} }} }                  	       	     
   	if(TimeDayOfWeek(IndiaDT) != 0 && TimeDayOfWeek(IndiaDT) != 6 
   	  && TimeHour(IndiaDT) >= LocalOpenHour && TimeHour(IndiaDT) < LocalCloseHour)
   	  {
    	  ObjectSetText( "z[CP Time] Indiat", Indias, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "z[CP Time] Indial", Indiap, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
      else
        {
    	  ObjectSetText( "z[CP Time] Indiat", Indias, ClockFontSize, FontName, Clock_MktClosed );     	
   	  ObjectSetText( "z[CP Time] Indial", Indiap, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Dubai == 1)   
      {
      if(!FLAG_match) { 
      if((DubaiDT <= BrokerDT+600 && DubaiDT >= BrokerDT-600)  &&
        (DubaiDT <= Local_Time+600 && DubaiDT >= Local_Time-600))
        {Dubaip= "Dubai  (B,L)";}           
      else {if ((DubaiDT <= BrokerDT+600) && (DubaiDT >= BrokerDT-600)) 
        {Dubaip= "Dubai  (B)";}
      else {if(DubaiDT == Local_Time) 
        {Dubaip= "Dubai  (L)";}                     
      else {Dubaip= "Dubai";} }} } 	     	      	       	     
   	if(TimeDayOfWeek(DubaiDT) != 0 && TimeDayOfWeek(DubaiDT) != 6 
   	  && TimeHour(DubaiDT) >= LocalOpenHour && TimeHour(DubaiDT) < LocalCloseHour)
        {
   	  ObjectSetText( "z[CP Time] Dubait", Dubais, ClockFontSize, FontName, Clock_MktOpen );     	
   	  ObjectSetText( "z[CP Time] Dubail", Dubaip, TimezoneFontSize, FontName, Location_MktOpen ); 
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Dubait", Dubais, ClockFontSize, FontName, Clock_MktClosed );  	   
   	  ObjectSetText( "z[CP Time] Dubail", Dubaip, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Moscow == 1)              
      {
      if(!FLAG_match) {  
      if((MoscowDT <= BrokerDT+600 && MoscowDT >= BrokerDT-600)  &&
        (MoscowDT <= Local_Time+600 && MoscowDT >= Local_Time-600))
        {Moscowp= "Moscow  (B,L)";}           
      else {if ((MoscowDT <= BrokerDT+600) && (MoscowDT >= BrokerDT-600)) 
        {Moscowp= "Moscow  (B)";}
      else {if(MoscowDT == Local_Time) 
        {Moscowp= "Moscow  (L)";}                      
      else {Moscowp= "Moscow";} }} }     	           	      	        	       
   	if(TimeDayOfWeek(MoscowDT) != 0 && TimeDayOfWeek(MoscowDT) != 6 
   	  && TimeHour(MoscowDT) >= LocalOpenHour && TimeHour(MoscowDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "z[CP Time] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "z[CP Time] Moscowl", Moscowp, TimezoneFontSize, FontName, Location_MktOpen ); 
        }
   	else
   	  {
    	  ObjectSetText( "z[CP Time] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktClosed );  	   
   	  ObjectSetText( "z[CP Time] Moscowl", Moscowp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Israel == 1)                          
      {
      if(!FLAG_match) {  
      if((IsraelDT <= BrokerDT+600 && IsraelDT >= BrokerDT-600)  &&
        (IsraelDT <= Local_Time+600 && IsraelDT >= Local_Time-600))
        {Israelp= "Israel (B,L)";}          
      else {if ((IsraelDT <= BrokerDT+600) && (IsraelDT >= BrokerDT-600)) 
        {Israelp= "Israel  (B)";}
      else {if(IsraelDT == Local_Time) 
        {Israelp= "Israel  (L)";}                  
      else {Israelp= "Israel";} }} }              	     	         	    
   	if(TimeDayOfWeek(IsraelDT) != 0 && TimeDayOfWeek(IsraelDT) != 6
   	  && TimeHour(IsraelDT) >= LocalOpenHour && TimeHour(IsraelDT) < LocalCloseHour)
        {
   	  ObjectSetText( "z[CP Time] Israelt", Israels, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "z[CP Time] Israell", Israelp, TimezoneFontSize, FontName, Location_MktOpen );
        }
   	else
        {
  	     ObjectSetText( "z[CP Time] Israelt", Israels, ClockFontSize, FontName, Clock_MktClosed );       	
   	  ObjectSetText( "z[CP Time] Israell", Israelp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      } 
    if(Helsinki == 1)
      {
      if(!FLAG_match) {                         
      if((HelsinkiDT <= BrokerDT+600 && HelsinkiDT >= BrokerDT-600)  &&
        (HelsinkiDT <= Local_Time+600 && HelsinkiDT >= Local_Time-600))
        {Helsinkip= "Helsinki  (B,L)";}         
      else {if((HelsinkiDT<=BrokerDT+600 && HelsinkiDT>=BrokerDT-600)) 
        {Helsinkip= "Helsinki  (B)";}
      else {if(HelsinkiDT == Local_Time) 
        {Helsinkip= "Helsinki  (L)";}                  
      else {Helsinkip= "Helsinki";} }} }	                             	       	                         	                             	                 
   	if(TimeDayOfWeek(HelsinkiDT) != 0 && TimeDayOfWeek(HelsinkiDT) != 6 
   	  && TimeHour(HelsinkiDT) >= LocalOpenHour && TimeHour(HelsinkiDT) < LocalCloseHour)
        {
   	  ObjectSetText( "z[CP Time] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "z[CP Time] Helsinkil", Helsinkip, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
    	  ObjectSetText( "z[CP Time] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktClosed );  	   
   	  ObjectSetText( "z[CP Time] Helsinkil", Helsinkip, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Frankfurt == 1)                         
      {
      if(!FLAG_match) {    
      if((BerlinDT <= BrokerDT+600 && BerlinDT >= BrokerDT-600)  &&
        (BerlinDT <= Local_Time+600 && BerlinDT >= Local_Time-600))
        {Berlinp= "Frankfurt  (B,L)";}         
      else {if ((BerlinDT <= BrokerDT+600) && (BerlinDT >= BrokerDT-600)) 
        {Berlinp= "Frankfurt  (B)";}
      else {if(BerlinDT == Local_Time) 
        {Berlinp= "Frankfurt  (L)";}               
      else {Berlinp= "Frankfurt";} }} }                          
   	if(TimeDayOfWeek(BerlinDT) != 0 && TimeDayOfWeek(BerlinDT) != 6 
   	  && TimeHour(BerlinDT) >= LocalOpenHour && TimeHour(BerlinDT) < LocalCloseHour)
        {
   	  ObjectSetText( "z[CP Time] Berlint", Berlins, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "z[CP Time] Berlinl", Berlinp, TimezoneFontSize, FontName, Location_MktOpen );
        }
   	else
        {
   	  ObjectSetText( "z[CP Time] Berlint", Berlins, ClockFontSize, FontName, Clock_MktClosed );     	
   	  ObjectSetText( "z[CP Time] Berlinl", Berlinp, TimezoneFontSize, FontName, Location_MktClosed ); 
        }
      } 
    if(London == 1)            
      {
      if(!FLAG_match) {     
      if((LondonDT <= BrokerDT+600 && LondonDT >= BrokerDT-600)  &&
        (LondonDT <= Local_Time+600 && LondonDT >= Local_Time-600))
        {Londonp= "London  (B,L)";}         
      else {if ((LondonDT <= BrokerDT+600) && (LondonDT >= BrokerDT-600)) 
        {Londonp= "London  (B)";}
      else {if(LondonDT == Local_Time) 
        {Londonp= "London  (L)";}               
      else {Londonp= "London";} }} //}
      //if (LondonDT == UTC + 3600) //test code during summer
      if(LondonDT == UTC)
        {
        if((Londonp== "London  (B)") || (Londonp== "London  (L)") || (Londonp== "London  (B,L)"))
          {
          Londonp= StringSubstr(Londonp,0,StringLen(Londonp)-1) + ",G)";             
          }
        else
          {
          Londonp= StringSubstr(Londonp,0) + "  (G)";
          }   
        }
      //Show London as B+1 if Broker is at GMT and GMT is not shown, and if London= GMT+1
      if((GMT == 0) && (LondonDT != UTC) && (FLAG2_match))
        {
        if(Londonp== "London  (L)")
          {
          Londonp= StringSubstr(Londonp,0,StringLen(Londonp)-1) + ",B+1)";             
          }
        if(Londonp== "London")
          {
          Londonp= Londonp + "  (B+1)";             
          }
        }  
      //Show London as G+1 if GMT not shown & Broker not at GMT, and if London= GMT+1
      if((GMT == 0) && (LondonDT != UTC) && (!FLAG2_match))
        {
        if(Londonp== "London  (L)")
          {
          Londonp= StringSubstr(Londonp,0,StringLen(Londonp)-1) + ",G+1)";             
          }
        if(Londonp== "London")
          {
          Londonp= Londonp + "  (G+1)";             
          }                      
        }}                                               
   	if(TimeDayOfWeek(LondonDT) != 0 && TimeDayOfWeek(LondonDT) != 6 
   	  && TimeHour(LondonDT) >= LocalOpenHour && TimeHour(LondonDT) < LocalCloseHour)
   	  {
        ObjectSetText( "z[CP Time] Londont", Londons, ClockFontSize, FontName, Clock_MktOpen );   	   
        ObjectSetText( "z[CP Time] Londonl", Londonp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Londont", Londons, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Londonl", Londonp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(GMT == 1)                
      {
      if(!FLAG_match) {        
      if((UTC <= BrokerDT+600 && UTC >= BrokerDT-600)  &&
        (UTC <= Local_Time+600 && UTC >= -600))
        {UTCp= "GMT  (B,L)";}           
      else {if ((UTC <= BrokerDT+600) && (UTC >= BrokerDT-600)) 
        {UTCp= "GMT  (B)";}     
      else {if (UTC== Local_Time) 
        {UTCp= "GMT  (L)";}                     	   
      else {UTCp= "GMT";} }} }     	              
   	if(TimeDayOfWeek(UTC) != 0 && TimeDayOfWeek(UTC) != 6 
   	  && TimeHour(UTC) >= LocalOpenHour && TimeHour(UTC) < LocalCloseHour)
   	  {
        ObjectSetText( "z[CP Time] utct", UTCs, ClockFontSize, FontName, Clock_MktOpen );   	   
        ObjectSetText( "z[CP Time] utcl", UTCp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {         
   	  ObjectSetText( "z[CP Time] utct", UTCs, ClockFontSize, FontName, Clock_MktClosed );                    	         
   	  ObjectSetText( "z[CP Time] utcl", UTCp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Brazil == 1)           
      {
      if(!FLAG_match) {  
      if((BrazilDT <= BrokerDT+600 && BrazilDT >= BrokerDT-600)  &&
        (BrazilDT <= Local_Time+600 && BrazilDT >= Local_Time-600))
        {Brazilp= "Brazil  (B,L)";}            
      else {if ((BrazilDT <= BrokerDT+600) && (BrazilDT >= BrokerDT-600)) 
        {Brazilp= "Brazil  (B)";}       
      else {if (BrazilDT == Local_Time) 
        {Brazilp= "Brazil  (L)";}       
      else {Brazilp= "Brazil";} }} }                    	     
   	if(TimeDayOfWeek(BrazilDT) != 0 && TimeDayOfWeek(BrazilDT) != 6 
   	  && TimeHour(BrazilDT) >= LocalOpenHour && TimeHour(BrazilDT) < LocalCloseHour)
   	  {
    	  ObjectSetText( "z[CP Time] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "z[CP Time] Brazill", Brazilp, TimezoneFontSize, FontName, Location_MktOpen );
        }
   	else
        {
   	  ObjectSetText( "z[CP Time] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktClosed );      	
   	  ObjectSetText( "z[CP Time] Brazill", Brazilp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(NewYork == 1)     
      {
      if(!FLAG_match) {       
      if((NewYorkDT <= BrokerDT+600 && NewYorkDT >= BrokerDT-600)  &&
        (NewYorkDT <= Local_Time+600 && NewYorkDT >= Local_Time-600))
        {NYp= "NewYork  (B,L)";}           
      else {if ((NewYorkDT <= BrokerDT+600) && (NewYorkDT >= BrokerDT-600)) 
        {NYp= "NewYork  (B)";}       
      else {if (NewYorkDT == Local_Time) 
        {NYp= "NewYork  (L)";}                         	   
      else {NYp= "NewYork";} }} }                         
   	if(TimeDayOfWeek(NewYorkDT) != 0 && TimeDayOfWeek(NewYorkDT) != 6 
   	  && TimeHour(NewYorkDT) >= LocalOpenHour && TimeHour(NewYorkDT) < LocalCloseHour)
   	  {
        ObjectSetText( "z[CP Time] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "z[CP Time] NewYorkl", NYp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
        ObjectSetText( "z[CP Time] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktClosed );  	   
        ObjectSetText( "z[CP Time] NewYorkl", NYp, TimezoneFontSize, FontName, Location_MktClosed ); 
        }
      }
    if(Central == 1)
      {             
      if(!FLAG_match) {
      if((CentralDT <= BrokerDT+600 && CentralDT >= BrokerDT-600)  &&
        (CentralDT <= Local_Time+600 && CentralDT >= Local_Time-600))
        {Centralp= "Central  (B,L)";}	          
      else {if ((CentralDT <= BrokerDT+600) && (CentralDT >= BrokerDT-600)) 
        {Centralp= "Central  (B)";}     
      else {if (CentralDT == Local_Time) 
        {Centralp= "Central  (L)";}      
      else {Centralp= "Central";} }} }                            	      
   	if(TimeDayOfWeek(CentralDT) != 0 && TimeDayOfWeek(CentralDT) != 6 
   	 && TimeHour(CentralDT) >= LocalOpenHour && TimeHour(CentralDT) < LocalCloseHour)
   	  {
  	     ObjectSetText( "z[CP Time] Centralt", Centrals, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "z[CP Time] Centrall", Centralp, TimezoneFontSize, FontName, Location_MktOpen ); 
   	  }
   	else
   	  {
  	     ObjectSetText( "z[CP Time] Centralt", Centrals, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Centrall", Centralp, TimezoneFontSize, FontName, Location_MktClosed ); 
   	  }
      }
    if(Mexico == 1)             
      {
      if(!FLAG_match) {
      if((MexicoDT <= BrokerDT+600 && MexicoDT >= BrokerDT-600)  &&
        (MexicoDT <= Local_Time+600 && MexicoDT >= Local_Time-600))
        {Mexicop= "Mexico  (B,L)";}        
      else {if ((MexicoDT <= BrokerDT+600) && (MexicoDT >= BrokerDT-600)) 
        {Mexicop= "Mexico  (B)";}      
      else {if (MexicoDT == Local_Time) 
        {Mexicop= "Mexico  (L)";}       
      else {Mexicop= "Mexico";} }} }                    	     
   	if(TimeDayOfWeek(MexicoDT) != 0 && TimeDayOfWeek(MexicoDT) != 6 
   	  && TimeHour(MexicoDT) >= LocalOpenHour && TimeHour(MexicoDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "z[CP Time] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "z[CP Time] Mexicol", Mexicop, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Mexicol", Mexicop, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Mountain == 1)         
      {
      if(!FLAG_match) { 
      if((MountainDT <= BrokerDT+600 && MountainDT >= BrokerDT-600)  &&
        (MountainDT <= Local_Time+600 && MountainDT >= Local_Time-600))
        {Mountainp= "Mountain  (B,L)";}           
      else {if ((MountainDT <= BrokerDT+600) && (MountainDT >= BrokerDT-600)) 
        {Mountainp= "Mountain  (B)";}      
      else {if (MountainDT == Local_Time) 
        {Mountainp= "Mountain  (L)";}        
      else {Mountainp= "Mountain";} }} }       	                 	      
   	if(TimeDayOfWeek(MountainDT) != 0 && TimeDayOfWeek(MountainDT) != 6 
   	  && TimeHour(MountainDT) >= LocalOpenHour && TimeHour(MountainDT) < LocalCloseHour)
        {
  	     ObjectSetText( "z[CP Time] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "z[CP Time] Mountainl", Mountainp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "z[CP Time] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "z[CP Time] Mountainl", Mountainp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Pacific == 1)                
      {
      if(!FLAG_match) { 
      if((PacificDT <= BrokerDT+600 && PacificDT >= BrokerDT-600)  &&
        (PacificDT <= Local_Time+600 && PacificDT >= Local_Time-600))
        {Pacificp= "Pacific  (B,L)";}           
      else {if ((PacificDT <= BrokerDT+600) && (PacificDT >= BrokerDT-600)) 
        {Pacificp= "Pacific  (B)";}      
      else {if(PacificDT == Local_Time) 
        {Pacificp= "Pacific  (L)";}     
      else {Pacificp= "Pacific";} }} }                       	     
   	if(TimeDayOfWeek(PacificDT) != 0 && TimeDayOfWeek(PacificDT) != 6 
   	  && TimeHour(PacificDT) >= LocalOpenHour && TimeHour(PacificDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "z[CP Time] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "z[CP Time] Pacificl", Pacificp, TimezoneFontSize, FontName, Location_MktOpen ); 
   	  }
   	else
        {
    	  ObjectSetText( "z[CP Time] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktClosed );     	
   	  ObjectSetText( "z[CP Time] Pacificl", Pacificp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Local == 1)        
      {
      if(!FLAG_match) {
      if(Local_Time == BrokerDT)  {Localp= "Local  (B)";}                          
      else {Localp= "Local";} 
      if(Local_Time <= UTC + 600  && Local_Time >= UTC - 600)
        {
        if(Localp== "Local  (B)") 
          {
          Localp= StringSubstr(Localp,0,StringLen(Localp)-1) + ",GMT)";     
          } 
        else
          {
          Localp= StringSubstr(Localp,0) + "  (GMT)";
          } 
        }}         
      ObjectSetText( "z[CP Time] Localt", Locals, ClockFontSize, FontName,Clock_MktClosed );      
      ObjectSetText( "z[CP Time] Locall", Localp, TimezoneFontSize, FontName, Location_MktClosed ); 
      }   
    if(!FLAG_match) 
      {  
      if(StaticBrokerDT < BrokerDT) {FLAG_match = true;} 
      }
    }//End Clock           
     
  //Bid Line----------------------------------------------------------------------------------- 
  if(Bid_Line_On)
    { 
    T2=Time[0];                  
          if(Chart_Zoom == 0) {T1=Time[0]+(Period()*60*3);}
    else {if(Chart_Zoom == 1) {T1=Time[0]+(Period()*60*10);}              
    else {if(Chart_Zoom == 2) {T1=Time[0]+(Period()*60*5);}   
    else {if(Chart_Zoom == 3) {T1=Time[0]+(Period()*60*3);}
    else {if(Chart_Zoom == 4) {T1=Time[0]+(Period()*60*2);}
    else {if(Chart_Zoom == 5) {T1=Time[0]+(Period()*60*1);}
    else {if(Chart_Zoom == 6) {T1=Time[0]+(Period()*60*1);} }}}}}}
    
    if(Period() == 43200) {T1=T1+(Period()*60*1);}
                    
    //Bid line   
    ObjectCreate(item18, OBJ_TREND, 0, T1, Bid, T2, Bid);     
    ObjectSet(item18, OBJPROP_STYLE, Bid_Ray_LineStyle_01234 );
    ObjectSet(item18, OBJPROP_COLOR, Bid_Ray_Color);
    ObjectSet(item18, OBJPROP_WIDTH, Bid_Ray_Thickness);               
    //Bid dot                   
    ObjectCreate(item19, OBJ_TREND, 0, T1, Bid, T1, Bid);     
    ObjectSet(item19, OBJPROP_STYLE, 0);
    ObjectSet(item19, OBJPROP_COLOR, Bid_Dot_Color);
    ObjectSet(item19, OBJPROP_WIDTH, Bid_Dot_Size);                      
    }//End Bid Line
   
  //Levels-------------------------------------------------------------------------------------
  if(Levels_On && Period()<= __Levels_Max_TF)
    {       
    //define needed variables
    ssp1= Bid/Poin;  //bid w/o the decimal 
    if(StringSubstr(Symbol(),0,6) == "XAUUSD") {ssp1= Bid;}           
    v1=ssp1%100; //last two significant digits   
    BL=v1; //last two significant digits of starting/reference bid line  
       
    //calculate line levels bottom to top
    for(j= -(100-(100-BL))-(100*NumberOfRanges); j<= (100-BL)+(100*NumberOfRanges); j++)       
      {
      ssp=ssp1+j; 
      v1=ssp%100;        
      if(v1==u1)
        {           
        linelevel = ssp*Poin;
        DrawLines("LVL", Levels_Display_Number, linelevel, Level_Whole_Color, 
        Level_Whole_Style_01234, Level_Whole_Width_12345);
        }
      else {if((__Show_Half_Lines) && (v1==u2))
        {              
        linelevel = ssp*Poin;
        DrawLines("LVL", Levels_Display_Number, linelevel, Level_Half_Color, 
        Level_Half_Style_01234, Level_Half_Width_12345);
        } 
      else {if((__Show_Quarter_Lines) && ((v1==u3) || (v1==u4)))
        {           
        linelevel = ssp*Poin;                        
        DrawLines("LVL", Levels_Display_Number, linelevel, Level_Quarter_Color, 
        Level_Quarter_Style_01234, Level_Quarter_Width_12345);
        } }}
      }
    }//End Levels         
                         
  //Daily Pivot Lines--------------------------------------------------------------------------  
  if((Pivots_PVT_On && Period()<= __PVT_Max_TF ) || 
    (Pivots_SR_On && Period()<= __SR_Max_TF))             
    {   
    //CaveMan's sophisticated Tz shift code finds out which hour bars start "today" and 
    //"yesterday" in the time zone selected for the pivots calculations.  The Tz code needs 
    //"tzdiff", which is the direction and number of hours offset from "Broker_GMT_Offset",
    //to start the process.  Unless the broker server is permanently at GMT, it will vary in
    //offset to GMT depending if it is the DST or non-DST time of the year.  Getting the 
    //proper DST/non-DST "Broker_GMT_Offset" is already handled by other coding. Specifically, 
    //the "tzdiff", is used by the Tz shift code to shift away from the broker server hour bar
    //that starts the current day, thus defining the start of the current time zone shifted
    //day.  Then this time zone shifted start of day is used to find the start and end of the 
    //previous time zone shifted day, which is the day used for the TzPivots calculations.
    
    //Define tzdiff
    if(!PivotsTz_On) {tzdiff = 0;}     
    else {tzdiff = Broker_GMT_Offset-__GMT_Offset;}

    //Send Items to the PivotsTz coding subroutine                       
    TzShift(tzdiff,idxfirstbartoday, idxfirstbaryesterday,idxlastbaryesterday);

    //Walk forward through yestday's start and collect high/lows within the same day
    yesterday_high= -99999;  // not high enough to remain alltime high
    yesterday_low=  +99999;  // not low enough to remain alltime low 
    for(idxbar= idxfirstbaryesterday; idxbar>=idxlastbaryesterday; idxbar--)
      {
      // grab first value for open
      if (yesterday_open==0) yesterday_open= iOpen(NULL, PERIOD_H1, idxbar);      
      yesterday_high= MathMax(iHigh(NULL, PERIOD_H1, idxbar), yesterday_high);
      yesterday_low= MathMin(iLow(NULL, PERIOD_H1, idxbar), yesterday_low);      
      yesterday_close= iClose(NULL, PERIOD_H1, idxbar);
      }
         
    d = (today_high - today_low);
    q = (yesterday_high - yesterday_low);
    p = (yesterday_high + yesterday_low + yesterday_close) / 3;                          
     
    if (Pivots_Fibonacci) {prefix = "F";}
    else {prefix = "D";} 
               
    if(Pivots_SR_On)
      {       
      if(Pivots_Fibonacci)
        {
        r5 = p+ (q * 2.618);
        r4 = p+ (q * 1.618);
  	     r3 = p+q;  
        r2 = p+ (q * 0.618); 
        r1 = p+ (q * 0.382); 
        s1 = p- (q * 0.382);     
        s2 = p- (q * 0.618);    
  	     s3 = p-q; 
        s4 = p- (q * 1.618);
        s5 = p- (q * 2.618);  
        }
      else   
        {
        r5 = (4*p)+(yesterday_high-(4*yesterday_low));
        r4 = (3*p)+(yesterday_high-(3*yesterday_low));
        r3 = (2*p)+(yesterday_high-(2*yesterday_low));
        r2 = p+(yesterday_high - yesterday_low);  //r2 = p-s1+r1;
        r1 = (2*p)-yesterday_low;  
        s1 = (2*p)-yesterday_high;
        s2 = p-(yesterday_high - yesterday_low);  //s2 = p-r1+s1;
        s3 = (2*p)-((2* yesterday_high)-yesterday_low);
        s4 = (3*p)-((3* yesterday_high)-yesterday_low);
        s5 = (4*p)-((4* yesterday_high)-yesterday_low);
        }                
      }
                      
    //Daily Central Pivot Line 
    if(Pivots_PVT_On && Period()<= __PVT_Max_TF )
      {         
      //Pivot Line data to subroutine                                                               
      DrawLines("PVT1", Pivot_PVT_Display_Number, p, PVT1_Back_Color, 
      PVT1_Style_01234, PVT1_Width_12345);
      DrawLines("PVT2", Pivot_PVT_Display_Number, p, PVT2_Top_Color, 
      PVT2_Style_01234, PVT2_Width_12345);         
      }
      
    //Daily Support & Resistance Lines   
    if(Pivots_SR_On && Period()<= __SR_Max_TF)
      {
      DrawLines(prefix+"R1", Pivot_SR_Display_Number, r1, R_Pivot_Color, 
      R_Style_01234, R_Width_12345);   
      DrawLines(prefix+"R2", Pivot_SR_Display_Number, r2, R_Pivot_Color, 
      R_Style_01234, R_Width_12345); 
      DrawLines(prefix+"S1", Pivot_SR_Display_Number, s1, S_Pivot_Color, 
      S_Style_01234, S_Width_12345);
      DrawLines(prefix+"S2", Pivot_SR_Display_Number, s2, S_Pivot_Color, 
      S_Style_01234, S_Width_12345);          
      if(__Show_Levels_Thru_2345 >= 3) {
      DrawLines(prefix+"R3", Pivot_SR_Display_Number, r3, R_Pivot_Color, 
      R_Style_01234, R_Width_12345);         
      DrawLines(prefix+"S3", Pivot_SR_Display_Number, s3, S_Pivot_Color, 
      S_Style_01234, S_Width_12345); }
      if(__Show_Levels_Thru_2345 >= 4) {
      DrawLines(prefix+"R4", Pivot_SR_Display_Number, r4, R_Pivot_Color, 
      R_Style_01234, R_Width_12345);          
      DrawLines(prefix+"S4", Pivot_SR_Display_Number, s4, S_Pivot_Color, 
      S_Style_01234, S_Width_12345); }
      if(__Show_Levels_Thru_2345 == 5) {           
      DrawLines(prefix+"R5", Pivot_SR_Display_Number, r5, R_Pivot_Color, 
      R_Style_01234, R_Width_12345);        
      DrawLines(prefix+"S5", Pivot_SR_Display_Number, s5, S_Pivot_Color, 
      S_Style_01234, S_Width_12345); }             
      }
    //Mid-Pivots
    if(__Show_MidPivots && Period()<= __SR_Max_TF) 
      {   
      DrawLines("MR1", Pivot_SR_Display_Number, (p+r1)/2,  MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);
      DrawLines("MR2", Pivot_SR_Display_Number, (r1+r2)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);
      DrawLines("MS1", Pivot_SR_Display_Number, (p+s1)/2,  MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);
      DrawLines("MS2", Pivot_SR_Display_Number, (s1+s2)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);
      if(__Show_Levels_Thru_2345 >= 3) {
      DrawLines("MR3", Pivot_SR_Display_Number, (r2+r3)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);     
      DrawLines("MS3", Pivot_SR_Display_Number, (s2+s3)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345); }  
      if(__Show_Levels_Thru_2345 >= 4) {      
      DrawLines("MR4", Pivot_SR_Display_Number, (r3+r4)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);             
      DrawLines("MR4", Pivot_SR_Display_Number, (s3+s4)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345); }
      if(__Show_Levels_Thru_2345 == 5) {       
      DrawLines("MR5", Pivot_SR_Display_Number, (r4+r5)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345);
      DrawLines("MS5", Pivot_SR_Display_Number, (s4+s5)/2, MidPivots_Color, 
      mP_Style_01234, mP_Width_12345); }
      }                             
    }//End Daily Pivots
    
  //Range Day High/Low lines-------------------------------------------------------------------        
 if(Range_Day_On && Period()<= __Day_Max_TF && Period()>= __Day_Min_TF)
    {
    //Note: Day range and bar data calculated in Panel section
    //Calculate position of RWH/RWL lines
    if(HiToday - LoToday <= dRange)
      {       
      dRangeHigh = LoToday + dRange;  
      dRangeLow  = HiToday - dRange; 
      }    
    else //if(HiToday - LoToday > dRange)
      {
      if(Time[Lbarshift] < Time[Hbarshift]){dRangeLow= LoToday; dRangeHigh= LoToday + dRange;}
      else {dRangeHigh= HiToday; dRangeLow= HiToday - dRange;}              
      }        
    //Range Lines data to subroutine      
    if(Period()>240 && Range_Day_Display_Number <3) {RDdisplay=3; RDshift=0;} 
    else {RDdisplay= Range_Day_Display_Number;}                                                                                    
    DrawLines("RDH", RDdisplay, dRangeHigh, RDH_Color, 
    RD_Style_01234, RD_Width_12345);   
    DrawLines("RDL", RDdisplay, dRangeLow,  RDL_Color, 
    RD_Style_01234, RD_Width_12345);         
    }//End Range Day
    
  //Range Week High/Low lines-----------------------------------------------------------------
  if(Range_Week_On && Period()<= __Week_Max_TF && Period()>= __Week_Min_TF)
    {
    //Define Week ATR
    wRange = iATR(NULL, 10080, Weeks_In_Week_ATR, 0);  
    //Define today's bar/data   
    HiWeek     = iHigh(NULL,PERIOD_W1,Weekshift);
	 LoWeek	   = iLow (NULL,PERIOD_W1,Weekshift);
	 wHbarshift = iHighest(NULL,0,MODE_HIGH,10080/Period(),Weekshift); 
	 wLbarshift = iLowest(NULL,0,MODE_LOW,10080/Period(),Weekshift); 
	 //Calculate position of RWH/RWL lines
	 if(HiWeek - LoWeek <= wRange)
      {     
      wRangeHigh = LoWeek + wRange;  
      wRangeLow  = HiWeek - wRange; 
      }        
    else //if(HiWeek - LoWeek > wRange)
      {
      if(Time[wLbarshift] < Time[wHbarshift]){wRangeLow= LoWeek; wRangeHigh= LoWeek + wRange;}
      else {wRangeHigh = HiWeek; wRangeLow = HiWeek - wRange;}        
      }   
    //Range Lines data to subroutine
    
    if(Period()>240 && Range_Week_Display_Number <3) {RWdisplay=3;}
    else {RWdisplay= Range_Week_Display_Number;}                                                               
    DrawLines("RWH", RWdisplay, wRangeHigh, RWH_Color, 
    RW_Style_01234, RW_Width_12345);   
    DrawLines("RWL", RWdisplay, wRangeLow,  RWL_Color, 
    RW_Style_01234, RW_Width_12345);
    }//End Range Week
       
  //Separators---------------------------------------------------------------------------------   
  if(Separators_On && Period() <= __Separators_Max_TF) 
    {
    //Separators for Today and Tomorrow           
    CurrentSeparators("Today", Separators_Color, Separators_Style_01234, 
    Separators_Width_12345); 
             
    CurrentSeparators("Tomorrow", Separators_Color, Separators_Style_01234, 
    Separators_Width_12345);  
              
    //Separators for prior sessions 
    if(__Prior_Days_To_Show > 0)
      {     
      //Calculate bars per session        
           if (Period()==1) {bps = 1440;}   
      else {if (Period()==5) {bps = 288;}  
      else {if (Period()==15){bps = 96;}    
      else {if (Period()==30){bps = 48;}
      else {if (Period()==60){bps = 24;}
      else {if (Period()==240){bps = 6;} }}}}}        
      //Define bar starting prior sessions display      
      shift = iBarShift(NULL,NULL,iTime(NULL,PERIOD_D1,0));                       
      //Execute loop for bars per session X number of prior sessions to show   
      for(i= shift; i<=(shift+(bps*(__Prior_Days_To_Show))); i++)   
        {    
        if(TimeHour(Time[i]) == 0 && TimeMinute(Time[i]) == 0)         
          {                     
          PriorSeparators(i, Separators_Color, 
          Separators_Style_01234, Separators_Width_12345);                             
          }         
        }               
      }
    }//End Separators   

  //vLines-------------------------------------------------------------------------------------
  if(vLines_On && Period()<= __Current_Session_Max_TF)
    {                 
    //Calculate position for vline labels------------------------------------------------------
    if(vLabels_On)
      {  
      if (vLabels_Dist_To_Border < 1) {vLabels_Dist_To_Border = 1;}           
      top = WindowPriceMax();
      bottom = WindowPriceMin();
      scale = top - bottom; 
      if(vLabels_Chart_Top) 
        {           
        YadjustTop = scale/(13000/FontSize);     
        YadjustTop = YadjustTop + (vLabels_Dist_To_Border * YadjustTop);                          	      	
        level = top - YadjustTop;
        }                 
      else if(!vLabels_Chart_Top) 
        {
        vLabels_Dist_Above_Border = vLabels_Dist_To_Border + 3;
        YadjustBot = scale/(500/FontSize); 
        YadjustBot = YadjustBot + ((vLabels_Dist_Above_Border * YadjustBot)/20);   
        level = bottom + YadjustBot;
        }  
      }
                  
    //Do vlines for current session------------------------------------------------------------    
    if(__Show_Session_Ao)
      {
      OpenToday(vLabels_Asian_Open, GMT_At_Ao + Broker_GMT_Offset, 
      vLines_Color, vLines_Style_01234, vLines_Width_12345, level); 
      }             
    OpenToday(vLabels_London_Open, GMT_At_Lo + Broker_GMT_Offset, 
    vLines_Color, vLines_Style_01234, vLines_Width_12345, level);
    OpenToday(vLabels_NewYork_Open, GMT_At_NYo + Broker_GMT_Offset,  
    vLines_Color,vLines_Style_01234, vLines_Width_12345, level);
    OpenToday(vLabels_London_Close, GMT_At_Lc + Broker_GMT_Offset, 
    vLines_Color, vLines_Style_01234, vLines_Width_12345, level);                         
                                                                      
    //Do vlines for previous sessions----------------------------------------------------------   
    if((__Prior_Sessions_To_Show >0) && (Period() <= __Prior_Sessions_Max_TF))
      {   
      //Calculate bars per session        
           if (Period()==1) {bps = 1440;}   
      else {if (Period()==5) {bps = 288;}  
      else {if (Period()==15){bps = 96;}    
      else {if (Period()==30){bps = 48;}
      else {if (Period()==60){bps = 24;} }}}}        
      //Define bar starting prior sessions display      
      shift = iBarShift(NULL,NULL,iTime(NULL,PERIOD_D1,0));               
      //Execute loop for bars per session X number of prior sessions to show    
      for(i= shift; i<=(shift+(bps*(__Prior_Sessions_To_Show))); i++)       
        {  
        h=TimeHour(Time[i]); 
        m=TimeMinute(Time[i]);
               
        //London Open vLine
        if(GMT_At_Lo+Broker_GMT_Offset<0) {GMT_At_Lo=GMT_At_Lo+24;}                
        MF = MathFloor(GMT_At_Lo); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, vLabels_London_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }               
        else {if((GMT_At_Lo != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, vLabels_London_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                   
        else {if((GMT_At_Lo == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))
          {
          OpenPrior(i, vLabels_London_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          } }}   
         
        //NewYork Open vLine             
        if(GMT_At_NYo+Broker_GMT_Offset<0) {GMT_At_NYo=GMT_At_NYo+24;}                
        MF = MathFloor(GMT_At_NYo); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, vLabels_NewYork_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                  
        else {if((GMT_At_NYo != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, vLabels_NewYork_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                   
        else {if((GMT_At_NYo == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))               
          {
          OpenPrior(i, vLabels_NewYork_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          } }}            
               
        //London Close vLine           
        if(GMT_At_Lc+Broker_GMT_Offset<0) {GMT_At_Lc=GMT_At_Lc+24;}                                                      
        MF = MathFloor(GMT_At_Lc); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, vLabels_London_Close, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                   
        else {if((GMT_At_Lc != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, vLabels_London_Close, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                   
        else {if((GMT_At_Lc == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))               
          {
          OpenPrior(i, vLabels_London_Close, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          } }}
          
        //Asian Open vLine
        if(__Show_Session_Ao) {           
        if(GMT_At_Ao+Broker_GMT_Offset<0) {GMT_At_Ao=GMT_At_Ao+24;}                                                      
        MF = MathFloor(GMT_At_Ao); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, vLabels_Asian_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                   
        else {if((GMT_At_Ao != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, vLabels_Asian_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          }                   
        else {if((GMT_At_Ao == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))               
          {
          OpenPrior(i, vLabels_Asian_Open, vLines_Color, vLines_Style_01234, 
          vLines_Width_12345, level);
          } }}}                                                                                    
        }//End prior sessions shift loop                            
      }//End prior sessions routine                        
    }//End vLines routine
  
   //End of program computations---------------------------------------------------------------        
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine To Compute Average Ranges                                            |                                                 
//+-------------------------------------------------------------------------------------------+ 
void Ranges (int period)
  {  
  int x, xx;
  ARg = 0;
  
  //Add ranges over period.  Count number of Sundays and exclude Sunday ranges.             
  for(i=1; i<=period; i++)
    {
    if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,i))!=0) {
    ARg = ARg + iHigh(NULL,PERIOD_D1,i)- iLow(NULL,PERIOD_D1,i);}
    else {x=x+1;}
    }

  //For number of Sundays, add additional days of range
  for(ii=i; ii<=i-1+x; ii++) 
    { 
    if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,ii))!=0) 
      {       
      ARg = ARg + iHigh(NULL,PERIOD_D1,ii)- iLow(NULL,PERIOD_D1,ii);
      }
    else {xx=xx+1;} 
    }
   
  //If a Sunday reduced added days above, add additional day of range
  for(iii=ii; iii<=ii-1+xx; iii++)
    {
    ARg = ARg + iHigh(NULL,PERIOD_D1,iii)- iLow(NULL,PERIOD_D1,iii);
    }
                        
  return (ARg);   
  }
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine To name and draw Daily Pivot and Range HL lines and line labels                |                                                                                  |
//+-------------------------------------------------------------------------------------------+
void DrawLines(string text, int Line_Selection_Number, double level, color Color, 
  int linestyle, int thickness)
  {
  //Lines====================================================================================== 
  //Lines & Labels - check values for Line Selection Number and Scale Shift
  if(Line_Selection_Number < 1 || Line_Selection_Number > 2) {Line_Selection_Number = 2;}

  //Name lines
  if(StringSubstr(text, 0, 2) == "RD" || StringSubstr(text, 0, 2) == "RW" ||
    StringSubstr(text, 0, 2) == "LV")
    {
    line = "[CP Draw]  _" + text + " Line " + DoubleToStr(level,Digits);  
    }        
  else {line = "[CP Draw]  " + text + " Line " + DoubleToStr(level,Digits);}  
    
  //Define variables  
  a = linestyle; b = thickness; c =1; if (a==0)c=b;  
   
  //Where to Start lines
  //start chart left
  if(Line_Selection_Number == 1) 
    {startline = WindowFirstVisibleBar();}
  //start at day separator 
  else if(Line_Selection_Number == 2) 
    {startline = iTime(NULL, PERIOD_D1, 0);}

                                                                              
  //Where to Stop Lines   
  //stop at chart right border
  if(Line_Selection_Number == 1) 
    {stopline = iTime(NULL,PERIOD_D1,0)+18748800; R2 = true;}
  //stop at end of current session
  else {if (Line_Selection_Number == 2) 
    {stopline = iTime(NULL,PERIOD_D1,0)+86400; R2 = false;}
  //stop at chart right border
  else {if (Line_Selection_Number == 3) {stopline = startline+ Period()*60; R2 = true;} }}  
                             
  //Draw lines   
  ObjectCreate(line, OBJ_TREND, 0, startline, level, stopline, level);    
  ObjectSet(line, OBJPROP_STYLE, a);
  ObjectSet(line, OBJPROP_COLOR, Color);
  ObjectSet(line, OBJPROP_WIDTH, c);
  if(Subordinate_Lines || StringSubstr(text,0,4) == "LVL") ObjectSet(line,OBJPROP_BACK,true); 
  else ObjectSet(line, OBJPROP_BACK, false);       
  ObjectSet(line, OBJPROP_RAY, R2);
   
  //Labels=====================================================================================
  //Exit if label not to be shown 
  if (StringSubstr(text, 0, 2) == "LV") {return (-1);}        
  else {if ((!Pivot_PVT_Label_On) && (StringSubstr(text, 0, 2) == "PV")) {return (-1);}
  else {if ((!Pivot_SR_Labels_On) && ((StringSubstr(text, 1, 1) == "R") || 
       (StringSubstr(text, 1, 1) == "S")) && (StringSubstr(text, 0, 1) != "M")) {return (-1);}
  else {if ((!Pivot_Mid_Labels_On) && (StringSubstr(text, 0, 1) == "M")) {return (-1);} 
  else {if ((!Range_Day_Labels_On) && (StringSubstr(text, 0, 2) == "RD")) {return (-1);}
  else {if ((!Range_Week_Labels_On) && (StringSubstr(text, 0, 2) == "RW")) {return (-1);}  
  else {if ((Range_Week_Labels_On) && (StringSubstr(text, 0, 2) == "RW") &&
       (!Range_Day_Labels_On) && (__Off_If_Day_Labels_Off) &&
       (Period() <= __Day_Max_TF)) {return (-1);}  
  else 
    {      
    //Name label 
    label = "[CP Draw]  " + text + " Label " + DoubleToStr(level,Digits);  
 
    //Pivots: Remove main pivot number
    if(StringSubstr(text, 0, 1) == "P") 
      {
      text = StringSubstr(text, 0, 3); 
      }
            
    //Pivot Labels: adjust for inclusion of "type" tags     
    if((StringSubstr(text, 0, 1) == "D") || (StringSubstr(text, 0, 1) == "F") || 
      (StringSubstr(text, 0, 1) == "M") || (StringSubstr(text, 0, 1) == "P"))
      {
      //Append asterisk when shifted pivots other than GMT+3 are displayed      
      if((PivotsTz_On) && (Pivots_PVT_On || Pivots_SR_On)) 
        {
        if(__GMT_Offset==3) {text = "   " + text + "  ";}
        else {text = "   " + text + "*";}
        }
      //Broker pivots append "^" tag 
      else {text = "   " + text + "^";}  
      }
                 
    //All other labels 
    else 
      {        
      text = "  " + text + " ";
      }     

    //Where to put labels
    if (Line_Selection_Number == 1)
      { 
      //start at mid-chart
      spc = ""; //00 
      startlabel = Time[WindowFirstVisibleBar()/2];           
      }
    else {if (Line_Selection_Number == 2)
      {            
      if(Time[WindowFirstVisibleBar()] < iTime(NULL, PERIOD_D1, 0)) 
        {
        //start at day separator if it is in chart window
        spc="        "; //08                           
        startlabel= iTime(NULL, PERIOD_D1, 0);  
        }          
      else      
        {
        //start at mid-chart
        spc = ""; //00 
        startlabel = Time[WindowFirstVisibleBar()/2];                                
        } 
      }                       
    else {if (Line_Selection_Number == 3)   
      {
      //start at current candle + Scale Shift to right 
      spc="         "; //09                   
      startlabel = Time[0] + T4; 
      } }}
                                
    //Draw labels                            
    ObjectCreate(label, OBJ_TEXT, 0, startlabel, level);     
    ObjectSetText(label, spc + text, FontSize, FontStyle, Labels_Color);
    if(Subordinate_Labels) ObjectSet(label,OBJPROP_BACK,true);   
    else ObjectSet(label, OBJPROP_BACK, false);
    } }}}}}}
  }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Today and Tomorrow Separators                                        |          
//+-------------------------------------------------------------------------------------------+   
void CurrentSeparators(string text, color Color, int linestyle, int thickness) 
  {
  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b; 
      
  //Define datetime for start of current session vline separator
  if(text == "Today")       
    { 
    dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + 
        TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + 0 + ":" + "0";                                        
    T3 = StrToTime(dt);           
    vline  = "[CP Draw] Current Session Line";     
    } 
      
  //Define datetime for end of current session vline separator
  else                
    {
    dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + 
        TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + 24 + ":" + "60";                                        
    T3 = StrToTime(dt)+60;                
    vline  = "[CP Draw] Tomorrow Session Line";            
    }
                        
  if(!Separators_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, T3, 0, T3, 100);}  
  else {ObjectCreate(vline, OBJ_VLINE, 0, T3, 0);}
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true); 
  }
      
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Prior Sessions Separators                                            |          
//+-------------------------------------------------------------------------------------------+   
void PriorSeparators(int i, color Color, int linestyle, int thickness)
  {
  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b;   
  vline  = "[CP Draw] Prior Session Line " + (i-1);       
                                 
  if(!Separators_Thru_SubWindows) 
    {ObjectCreate(vline, OBJ_TREND, 0, Time[i], 0, Time[i], 100);} 
  else {ObjectCreate(vline, OBJ_VLINE, 0, Time[i], 0);}     
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true);  
  } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Market Open Lines And Labels For Current Session                     |          
//+-------------------------------------------------------------------------------------------+   
void OpenToday(string text,int time,color Color,int linestyle,int thickness,double level) 
  {
  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b;

  vline  = "[CP Draw]  " + StringTrimLeft(text) + " Current Session Line";    
  vlabel = "[CP Draw]  " + StringTrimLeft(text) + " Current Session Label";   
  time2 = time;
  if(time2<0) {time2 = time2 + 24;}
  time3 = MathFloor(time2);             
  dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + 
      TimeDay(iTime(NULL,PERIOD_D1,0))+ "." +time3+":"+"0";           
  time4 = StrToTime(dt);      
  if(time2 != time3) {time4 = time4 + 1800;}  //add half hour  
                         
  if(!vLines_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, time4, 0, time4, 100);}  
  else {ObjectCreate(vline, OBJ_VLINE, 0, time4, 0);}
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true); 

  //Draw labels  
  if(vLabels_On)  
    {                                   
    ObjectCreate (vlabel, OBJ_TEXT, 0, time4, level);
    ObjectSetText(vlabel, text, vFontSize, vFontStyle, vLabels_Color);
    ObjectSet(vlabel, OBJPROP_BACK, false);      
    }
  }

//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Market Open Lines And Labels For Prior Sessions                      |          
//+-------------------------------------------------------------------------------------------+   
void OpenPrior(int i, string text, color Color, int linestyle, int thickness, double level) 
  {   
  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b;
  
  vline  = "[CP Draw]  " + StringTrimLeft(text) + " Prior Session Line " + (i-1);    
  vlabel = "[CP Draw]  " + StringTrimLeft(text) + " Prior Session Label " + (i-1); 
                
  //ObjectDelete(vline); 
  if(!vLines_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, Time[i], 0, Time[i], 100);} 
  else {ObjectCreate(vline, OBJ_VLINE, 0, Time[i], 0);}     
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true); 
  
  //Draw labels  
  if(vLabels_On) 
    {                                           
    ObjectCreate (vlabel, OBJ_TEXT, 0, Time[i], level);
    ObjectSetText(vlabel, text, vFontSize, vFontStyle, vLabels_Color);
    ObjectSet(vlabel, OBJPROP_BACK, false);         
    }    
  }

//+-------------------------------------------------------------------------------------------+
//| TzShift sub-routine computes index of first/last bars of yesterday & first bar  of today  |
//+-------------------------------------------------------------------------------------------+
void TzShift(int &tzdiff,int &idxfirstbartoday,int &idxfirstbaryesterday,int &idxlastbaryesterday)              
  {      
  tzdiffsec            = tzdiff*3600; 
  barsperday           = (1440/PERIOD_H1);      
  dayofweektoday       = TimeDayOfWeek(iTime(NULL, PERIOD_H1, 0) - tzdiffsec);
  dayofweektofind      = -1;     
           
  //Due to gaps in the data, and shift of time around weekends (due to time zone)
  //it is not as easy as to just look back for a bar with 00:00 time  
  idxfirstbartoday= 0; idxfirstbaryesterday= 0;  idxlastbaryesterday= 0;     
  switch (dayofweektoday) { 
     case 0: dayofweektofind = 5; break; // Sun (Sunday considered as short trading day)
     default: dayofweektofind = dayofweektoday -1; break;}                    

  //Search backwards for the last occurence of a "today's first bar"
  for(int i=1; i<=barsperday + 1; i++){
    datetime timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
      if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
  for(int j= 0; j<=2*barsperday + 1; j++) {
    datetime timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
    if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
  for(j= 1; j<=barsperday; j++) {
    datetime timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
    if(TimeDayOfWeek(timey2) != dayofweektofind) 
      {idxfirstbaryesterday = idxlastbaryesterday+j-1; break;}}
      
//Start - Added by CaveMan to cater for missing/partial previous day's data        
  //Search for Friday's data range if yesterday is a Sunday  
  if(dayofweektofind == 0)
    {
    //Search backwards for the last occurence of a "today's first bar"
    for(i=1; i<=barsperday + 1; i++){
       timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
       if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
    for(j= 0; j<=2*barsperday + 1; j++) {
       timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
       if(TimeDayOfWeek(timey) == 5) {idxlastbaryesterday = i+j; break;}}
    for(j= 1; j<=barsperday; j++) {
       timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
       if(TimeDayOfWeek(timey2) != 5){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
    } 
                      
  //This section takes care of the Christmas and New Year Day problem where there is a day 
  //of partial data on the eve of these holidays followed by an entire day without any data  
  //at all. The workaround to handle this on market open after the holiday is to use data 
  //from the day before the eve of the holiday right up to the end of the eve of the holiday 
  //for pivot computation. E.g. On the 26 December market opens after Christmas. Use full 
  //day data of 23rd plus partial data of 24th Dec to compute the pivots. Same applies to 
  //New Year Day.
  int YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
  if(YesterdayBarsTotal == 0)   
    {
    if (dayofweektofind == 0) {dayofweektofind = 4;} 
    else if (dayofweektofind == 1) {dayofweektofind = 5;} 
    else {dayofweektofind = dayofweektofind - 1;}

    //Repeat this search with the new dayofweektofind information
    for(i=1; i<=barsperday + 1; i++) {
       timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
       if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
    for(j= 0; j<=2*barsperday + 1; j++) {
       timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
       if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
    for(j= 1; j<=3*barsperday; j++) {
       timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
       if(TimeDayOfWeek(timey2) != dayofweektofind) 
         {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
    YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
    }            
  //Standard codes to handle days of partial data (including Fridays and Sundays)
  //If one hour bars < 24, use additional data from one more day back 
  if(YesterdayBarsTotal < 23)
    {
    //for Sunday, search Thursday and found Thursday partial data, then search Wednesday. 
    if (dayofweektoday == 0 && dayofweektofind == 4) {dayofweektofind = 3;} 
    //for Monday, Friday no data then include Thursday and Sunday data.
    else if (dayofweektoday == 1 && dayofweektofind == 4) {dayofweektofind = 4;} 
    else if (dayofweektofind == 0 || dayofweektofind == 1) { dayofweektofind = 5;} 
    else {dayofweektofind = dayofweektofind - 1;}
    //Search backwards for the last occurence of a "today's first bar"
    for(i=1; i<=barsperday + 1; i++) {
      timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
      if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
    for(j= 0; j<=3*barsperday + 1; j++) { // Need to increase another 24 hours to search
      timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
      if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
    for(j= 1; j<=4*barsperday; j++) {
      timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
      if(TimeDayOfWeek(timey2) != dayofweektofind)
        {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}                             
    //This small section handles a very unique combination of events that coincide during the 
    //short opening trading hours of the Sunday following Christmas Day and New Year Day if 
    //both of these holidays happen to fall on a Thursday.  This occured on the short Sunday 
    //trading hours before Monday kicks in. For those few hours, pivot used on Friday are 
    //extended into Sunday. These pivot levels are calculated using full Thursday's data. In 
    //this unique situation, Thursday was New Year Day (major public holiday)with no data at 
    //all. This will cause TzPivot to malfunction in similar situation in the future. As a 
    //workaround, for those short few hours of Sunday, if a similar situation should occur,
    //partial data of Wednesday and Friday will be used for pivot computation.  
    YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
    if(YesterdayBarsTotal == 0)   
      {
      if(dayofweektofind == 1) {dayofweektofind = 5;}
      else{dayofweektofind = dayofweektofind - 1;}
      //Repeat this search with the new dayofweektofind information
      for(i=1; i<=barsperday + 1; i++) {
        timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
        if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
      for(j= 0; j<=2*barsperday + 1; j++) {
        timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
        if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
      for(j= 1; j<=3*barsperday; j++) {
        timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
        if(TimeDayOfWeek(timey2) != dayofweektofind)
          {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
      idxlastbaryesterday = idxfirstbartoday + 1;
         }                     
    //Include partial day's data and combine with previous day's data for pivot computation. 
    //Do not include for Sundays as Sunday short trading hours uses Friday's pivots that are 
    //computed from full Thursday's data.  This is more for handling of holidays including 
    //Christmas and New Year Day. . 
    if (dayofweektoday != 0) idxlastbaryesterday = idxfirstbartoday + 1; 
    }          
//End - Added by CaveMan to cater for missing/partial previous day's data 
  }                                

//+-------------------------------------------------------------------------------------------+
//| Subroutine to make city and time labels                                                   |
//+-------------------------------------------------------------------------------------------+
int MakeLabel( string n, int xoff, int yoff )
  {
  if(ObjectFind(n) != 0)
    {    
    ObjectCreate( n, OBJ_LABEL, 0, 0, 0 );
    ObjectSet( n, OBJPROP_CORNER, 0 );
    ObjectSet( n, OBJPROP_XDISTANCE, xoff );
    ObjectSet( n, OBJPROP_YDISTANCE, yoff );
    ObjectSet( n, OBJPROP_BACK, false ); 
    }
  else{ObjectMove(n, 0, xoff, yoff);}       
  } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to convert time to a string for time labels                                    |
//+-------------------------------------------------------------------------------------------+
string Time_To_String( datetime when )
   {
   hour = TimeHour( when );
   if ( !Show_AMPM_Time ) 
      {
      timeStr = (TimeToStr( when, TIME_MINUTES));
      }

   // User wants 12HourTime format with "AM" or "PM".   
   // FYI, if >12:00, subtract 12 hours in seconds which is 12*60*60=43200
   else
      {
      if ( hour > 12 || hour == 0) timeStr = TimeToStr( (when - 43200), TIME_MINUTES);
      else timeStr = TimeToStr( when, TIME_MINUTES);
      if ( hour >= 12) timeStr = StringConcatenate(timeStr, " PM");
      else timeStr = StringConcatenate(timeStr, " AM");
      }
   return (timeStr);
   }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to retrieve date and time from time array                                      |
//+-------------------------------------------------------------------------------------------+
datetime TimeArrayToTime(int& LocalTimeArray[])
  {  
  nYear=LocalTimeArray[0]&0x0000FFFF;
  nMonth=LocalTimeArray[0]>>16;
  nDay=LocalTimeArray[1]>>16;
  nHour=LocalTimeArray[2]&0x0000FFFF;
  nMin=LocalTimeArray[2]>>16;
  nSec=LocalTimeArray[3]&0x0000FFFF;
  nMilliSec=LocalTimeArray[3]>>16;
  LocalTimeS = FormatDateTime(nYear,nMonth,nDay,nHour,nMin,nSec);
  Local_Time = StrToTime( LocalTimeS );
  return(Local_Time);
  }   

//+-------------------------------------------------------------------------------------------+
//| Subroutine to format time labels                                                          |
//+-------------------------------------------------------------------------------------------+
string FormatDateTime(int nYear,int nMonth,int nDay,int nHour,int nMin,int nSec)
  {
  sMonth=100+nMonth;
  sMonth=StringSubstr(sMonth,1);
  sDay=100+nDay;
  sDay=StringSubstr(sDay,1);
  sHour=100+nHour;
  sHour=StringSubstr(sHour,1);
  sMin=100+nMin;
  sMin=StringSubstr(sMin,1);
  sSec=100+nSec;
  sSec=StringSubstr(sSec,1);
  return(StringConcatenate(nYear,".",sMonth,".",sDay," ",sHour,":",sMin,":",sSec));
  }    
//+-------------------------------------------------------------------------------------------+
//| Subroutine to get time zone information for each city                                     |
//+-------------------------------------------------------------------------------------------+
void Get03TimeZoneInfo(int& SydneyTZInfoArray[], int& LondonTZInfoArray[], int& NewYorkTZInfoArray[])
  
   {
   SydneyTZInfoArray[0]  = -600;
   SydneyTZInfoArray[17] = 262144; //4<<16 == 262144  April
   SydneyTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   SydneyTZInfoArray[19] = 3;
   SydneyTZInfoArray[20] = 0;
   SydneyTZInfoArray[21] = 0;
   SydneyTZInfoArray[38] = 655360; //10<<16 == 655360  October
   SydneyTZInfoArray[39] = 65536;  //1<<16 == 65536    1st Sunday
   SydneyTZInfoArray[40] = 2;
   SydneyTZInfoArray[41] = 0;
   SydneyTZInfoArray[42] = -60;
     
   LondonTZInfoArray[0]  = 0;
   LondonTZInfoArray[17] = 655360; //10<<16 == 655360  October
   LondonTZInfoArray[18] = 327680; //5<<16 == 327680.  5th/Last Sunday. 
   LondonTZInfoArray[19] = 2;
   LondonTZInfoArray[20] = 0;
   LondonTZInfoArray[21] = 0;
   LondonTZInfoArray[38] = 196608; //3<<16 == 196608  March
   LondonTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   LondonTZInfoArray[40] = 1;
   LondonTZInfoArray[41] = 0;
   LondonTZInfoArray[42] = -60;
   
   NewYorkTZInfoArray[0]  = 300;
   NewYorkTZInfoArray[17] = 720896; //wYear = 0. wMonth = 11, and 11<<16 == 720896
   NewYorkTZInfoArray[18] = 65536;  //wDOW = 0 = Sunday. nDay = 1 and 1<<16 == 65536 
   NewYorkTZInfoArray[19] = 2;
   NewYorkTZInfoArray[20] = 0;
   NewYorkTZInfoArray[21] = 0;
   NewYorkTZInfoArray[38] = 196608; //3<<16 == 196608  March
   NewYorkTZInfoArray[39] = 131072; //2<<16 == 131072  2nd Sunday
   NewYorkTZInfoArray[40] = 2;
   NewYorkTZInfoArray[41] = 0;
   NewYorkTZInfoArray[42] = -60;
   }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to get time zone information for each city                                     |
//+-------------------------------------------------------------------------------------------+
void Get15TimeZoneInfo(int& AucklandTZInfoArray[], 
   int& TokyoTZInfoArray[], int& ChinaTZInfoArray[], int& JakartaTZInfoArray[], 
   int& IndiaTZInfoArray[], int& DubaiTZInfoArray[], int& MoscowTZInfoArray[], 
   int& IsraelTZInfoArray[], int& HelsinkiTZInfoArray[], int& BerlinTZInfoArray[], 
   int& BrazilTZInfoArray[], 
   int& CentralTZInfoArray[], int& MexicoTZInfoArray[], int& MountainTZInfoArray[], 
   int& PacificTZInfoArray[])
   
   {
   AucklandTZInfoArray[0]  = -720;
   AucklandTZInfoArray[17] = 262144; //4<<16 == 262144  April
   AucklandTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   AucklandTZInfoArray[19] = 3;
   AucklandTZInfoArray[20] = 0;
   AucklandTZInfoArray[21] = 0;
   AucklandTZInfoArray[38] = 589824; //9<<16 == 589824  September
   AucklandTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   AucklandTZInfoArray[40] = 2;
   AucklandTZInfoArray[41] = 0;
   AucklandTZInfoArray[42] = -60;
   
   // FYI Tokyo = Seoul
   TokyoTZInfoArray[0]  = -540;
   TokyoTZInfoArray[17] = 0; 
   TokyoTZInfoArray[18] = 0;
   TokyoTZInfoArray[19] = 0;
   TokyoTZInfoArray[20] = 0;
   TokyoTZInfoArray[21] = 0;
   TokyoTZInfoArray[38] = 0;
   TokyoTZInfoArray[39] = 0;
   TokyoTZInfoArray[40] = 0;
   TokyoTZInfoArray[41] = 0;
   TokyoTZInfoArray[42] = 0;
   
   // FYI, Beijing = Perth, Singapore, Taipei, Hong Kong
   ArrayCopy(ChinaTZInfoArray, TokyoTZInfoArray);
   ChinaTZInfoArray[0] = -480;
   
   // FYI, Jakarta = Bangkok
   ArrayCopy(JakartaTZInfoArray, TokyoTZInfoArray);
   JakartaTZInfoArray[0] = -420;
   
   ArrayCopy(IndiaTZInfoArray, TokyoTZInfoArray);
   IndiaTZInfoArray[0]   = -330;  // NOTE! Top of the hour is 30 min off most world timezones.
   
   ArrayCopy(DubaiTZInfoArray, TokyoTZInfoArray);
   DubaiTZInfoArray[0]   = -240;
 
   IsraelTZInfoArray[0]  = -120;
   IsraelTZInfoArray[17] = 589824;  //9<<16 == 589824  September
   IsraelTZInfoArray[18] = 131072;  //2<<16  = 131072
   IsraelTZInfoArray[19] = 2;
   IsraelTZInfoArray[20] = 0;
   IsraelTZInfoArray[21] = 0;
   IsraelTZInfoArray[38] = 196608; //3<<16 == 196608  March
   IsraelTZInfoArray[39] = 327685; //5<<16  = 327680 AND "5" (*IF* Sunday is "0", is "5" Friday??)
   IsraelTZInfoArray[40] = 2;
   IsraelTZInfoArray[41] = 0;
   IsraelTZInfoArray[42] = -60;
   
   ArrayCopy(MoscowTZInfoArray, LondonTZInfoArray);
   MoscowTZInfoArray[0]  = -180;
   MoscowTZInfoArray[19] = 3;
   MoscowTZInfoArray[40] = 2;
   
   // FYI, Helsinki = Athens
   ArrayCopy(HelsinkiTZInfoArray, LondonTZInfoArray);
   HelsinkiTZInfoArray[0]  = -120;
   HelsinkiTZInfoArray[19] = 4;
   HelsinkiTZInfoArray[40] = 3;
   
   // FYI, Berlin = Belgrade, Brussels, Paris, Sarajevo
   ArrayCopy(BerlinTZInfoArray, MoscowTZInfoArray);
   BerlinTZInfoArray[0] = -60;
   
   // NOTE! Brazil's ST/DST is likely a hardcoded date which will need updating EVERY YEAR!
   BrazilTZInfoArray[0]  = 180;
   BrazilTZInfoArray[17] = 131072;   //2<<16  = 131072
   BrazilTZInfoArray[18] = 196614;   //3<<16 == 196608 AND 6, so, Saturday?? or Mar 6?
   BrazilTZInfoArray[19] = 3866647;
   BrazilTZInfoArray[20] = 65470523;
   BrazilTZInfoArray[21] = 0;
   BrazilTZInfoArray[38] = 655360;   //10<<16 == 655360
   BrazilTZInfoArray[39] = 196614;
   BrazilTZInfoArray[40] = 3866647;
   BrazilTZInfoArray[41] = 65470523;
   BrazilTZInfoArray[42] = -60;
   
   //ArrayCopy(BuenasAriesTZInfoArray, BuenasAriesTZInfoArray);
   //BuenasAriesTZInfoArray[0] = 180;
   
   ArrayCopy(CentralTZInfoArray, NewYorkTZInfoArray);
   CentralTZInfoArray[0] = 360;
 
   MexicoTZInfoArray[0]  = 360;
   MexicoTZInfoArray[17] = 655360; //10<<16 == 655360  October
   MexicoTZInfoArray[18] = 327680; //5<<16 == 327680  5th/Last Sunday
   MexicoTZInfoArray[19] = 2;
   MexicoTZInfoArray[20] = 0;
   MexicoTZInfoArray[21] = 0;
   MexicoTZInfoArray[38] = 262144; //4<<16 == 262144  April
   MexicoTZInfoArray[39] = 65536;  //1<<16 == 65536   1st Sunday
   MexicoTZInfoArray[40] = 2;
   MexicoTZInfoArray[41] = 0;
   MexicoTZInfoArray[42] = -60;
   
   ArrayCopy(MountainTZInfoArray, NewYorkTZInfoArray);
   MountainTZInfoArray[0] = 420;
   
   ArrayCopy(PacificTZInfoArray, NewYorkTZInfoArray);
   PacificTZInfoArray[0] = 480;   
   } 
   
//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+      

