//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                  SonicR Trade Levels.mq4                                  | 
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2013 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview, _v1:

This indicator shows labeled lines on the chart for those EP (Entry), TP (Take Profit) and SL
(Stop Loss) trade prices that you enter into External Inputs.  You can enter up to twenty EPs
three TPs and three SLs. With more than one EP, an average EP is displayed and can be shifted 
to the right if the group of EPs is so close together that values overlap the average EP. 
When the average EP is right shifted, TPs will automatically be right shifted also.

Please note that you must enter EPs sequentially, without skipping a number and leaving it as
a "0" between other inputs with non-zero values, as this will result in an average calculation
that is incorrect and a display of the average with the incorrect number of averaged entries.

Placement of the end of lines and the start of labels requires different hard code values for
for each chart zoom setting (1-6).  Determination of the chart zoom setting is derived from  
the SonicR Filled Dragon indicator, or if not on the chart then from the SonicR PVA Candles
indicator.  One or the other must be attached to the chart, even if not turned on, so that as
you zoom in/out on a chart the selection of the appropriate values is handled automatically.
If both indicators are on the chart list of indicators, but are set to different chart zoom 
settings, the last indicator on the list of indicators is the controlling indicator.  

An optional alert triggers each time the bid price either equals or crosses the "__Bid_Price" 
you input.  You can set this alert to trigger only once per TF to reduce the number of alerts.  
You might use this feature to alert you to when price is approaching an area where you wish to 
consider adding to your trade, closing the trade, or adjusting your SL. 

A single buffer with no color is included.  This utility comes into play when changing chart
Properties to "Scale fix" (checked).  Sometimes when doing this, a colored line appears on the
right chart border.  This  buffer prevents that line from appearing.  This utility works only
if "Chart on foreground" is checked in chart "Properties/Common tab", and if this indicator is 
the last indicator listed for the body of the chart.

                                                                   - Traderathome, 05-25-2013  
-----------------------------------------------------------------------------------------------
Suggested Colors:            White Chart        Black Chart      

EP_Color                     MediumBlue         C'091,142,236' 
EP_Av_Color                  DarkOrchid         MediumOrchid;
SL_Color                     C'213,0,0'         C'226,035,035'  
TP_Color                     C'0,128,75'        C'016,186,092'             
---------------------------------------------------------------------------------------------*/

//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window 

//Fix Chart Scale
#property indicator_buffers  1
#property indicator_color1   CLR_NONE  

//Global External Inputs
extern bool   Indicator_On                    = false;
extern bool   Alert_on_Price                  = false;
extern double __Bid_Price                     = 0;
extern bool   __Alert_Only_Once_Per_TF        = false; 
extern string Broker_Name_In_Alert            = "";
extern bool   Right_Shift_Avg_EP_and_TPs      = true;
extern double EP_1                            = 0;
extern double EP_2                            = 0;
extern double EP_3                            = 0;
extern double EP_4                            = 0;
extern double EP_5                            = 0;
extern double EP_6                            = 0;
extern double EP_7                            = 0;
extern double EP_8                            = 0;
extern double EP_9                            = 0;
extern double EP_10                           = 0;
extern double EP_11                           = 0;
extern double EP_12                           = 0;
extern double EP_13                           = 0;
extern double EP_14                           = 0;
extern double EP_15                           = 0;
extern double EP_16                           = 0;
extern double EP_17                           = 0;
extern double EP_18                           = 0;
extern double EP_19                           = 0;
extern double EP_20                           = 0;
extern double SL_1                            = 0;
extern double SL_2                            = 0;
extern double SL_3                            = 0;
extern double TP_1                            = 0;
extern double TP_2                            = 0;
extern double TP_3                            = 0;
extern color  EP_Color                        = MediumBlue;  
extern color  EP_Av_Color                     = DarkOrchid;
extern color  SL_Color                        = C'213,0,0';  
extern color  TP_Color                        = C'0,128,75';
extern int    Display_Min_TF                  = 1;
extern int    Display_Max_TF                  = 43200;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

//Buffers, Constants and Variables
bool          FLAG_deinit;
double        Avg_Price;
int           k,obj_total,S1,S2,T1,T2,T3,T4,T5,Trades;
string        EPstr,name;
string        spc = "                                  "; //34 
string        name01 = "[Trade] EP1 Line";
string        name02 = "[Trade] EP1 Label";
string        name03 = "[Trade] EP2 Line";
string        name04 = "[Trade] EP2 Label";
string        name05 = "[Trade] EP3 Line";
string        name06 = "[Trade] EP3 Label";
string        name07 = "[Trade] EP4 Line";
string        name08 = "[Trade] EP4 Label";
string        name09 = "[Trade] EP5 Line";
string        name10 = "[Trade] EP5 Label";
string        name11 = "[Trade] EP6 Line";
string        name12 = "[Trade] EP6 Label";
string        name13 = "[Trade] EP7 Line";
string        name14 = "[Trade] EP7 Label";
string        name15 = "[Trade] EP8 Line";
string        name16 = "[Trade] EP8 Label";
string        name17 = "[Trade] EP9 Line";
string        name18 = "[Trade] EP9 Label";
string        name19 = "[Trade] EP10 Line";
string        name20 = "[Trade] EP10 Label";
string        name21 = "[Trade] EP11 Line";
string        name22 = "[Trade] EP11 Label";
string        name23 = "[Trade] EP12 Line";
string        name24 = "[Trade] EP12 Label";
string        name25 = "[Trade] EP13 Line";
string        name26 = "[Trade] EP13 Label";
string        name27 = "[Trade] EP14 Line";
string        name28 = "[Trade] EP14 Label";
string        name29 = "[Trade] EP15 Line";
string        name30 = "[Trade] EP15 Label";
string        name31 = "[Trade] EP16 Line";
string        name32 = "[Trade] EP16 Label";
string        name33 = "[Trade] EP17 Line";
string        name34 = "[Trade] EP17 Label";
string        name35 = "[Trade] EP18 Line";
string        name36 = "[Trade] EP18 Label";
string        name37 = "[Trade] EP19 Line";
string        name38 = "[Trade] EP19 Label";
string        name39 = "[Trade] EP20 Line";
string        name40 = "[Trade] EP20 Label";
string        name51 = "[Trade] Avg Line";
string        name52 = "[Trade] Avg Label";
string        name61 = "[Trade] TP1 Line";
string        name62 = "[Trade] TP1 Label";
string        name63 = "[Trade] TP2 Line";
string        name64 = "[Trade] TP2 Label";
string        name65 = "[Trade] TP3 Line";
string        name66 = "[Trade] TP3 Label";
string        name71 = "[Trade] SL1 Line";
string        name72 = "[Trade] SL1 Label";
string        name73 = "[Trade] SL2 Line";
string        name74 = "[Trade] SL2 Label";
string        name75 = "[Trade] SL3 Line";
string        name76 = "[Trade] SL3 Label";

//Alert-
bool   Alert_Allowed;   
double prevBid;

//Labels
string Label_Style                     = "Arial Bold";  
int    Label_Size                      = 9;

//Global Variable Controlling Bar Widths
double Chart_Zoom; 
string Zoom = "Zoom_Setting";

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+ 
int init()
  {
  FLAG_deinit = false;
  
  //Get GV Bar Widths for candle bars
  Chart_Zoom = GlobalVariableGet(Zoom); 
  
  //Alert
  if(Alert_on_Price == true) {Alert_Allowed = true;} 
  prevBid = 0; 
  
  //Find average
    {    
    if(EP_20 > 0) 
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20)/20;
      Trades = 20;     
      }       
    else {if(EP_19 > 0) 
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19)/19;
      Trades = 19;     
      }       
    else {if(EP_18 > 0) 
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18)/18;
      Trades = 18;     
      }    
    else {if(EP_17 > 0) 
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17)/17;
      Trades = 17;     
      }
    else {if(EP_16 > 0)      
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16)/16;
      Trades = 16;   
      }    
    else {if(EP_15 > 0)      
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14 + EP_15)/15;
      Trades = 15;        
      }    
    else {if(EP_14 > 0)      
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13 + EP_14)/14;
      Trades = 14;        
      }      
    else {if(EP_13 > 0)      
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12 + EP_13)/13;
      Trades = 13;        
      }    
    else {if(EP_12 > 0)      
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11 + EP_12)/12;
      Trades = 12;                               
      }
    else {if(EP_11 > 0)       
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
      + EP_11)/11;
      Trades = 11;                               
      }   
    else {if(EP_10 > 0)       
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10)/10;
      Trades = 10;                               
      }   
    else {if(EP_9 > 0)       
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9)/9;
      Trades = 9;                               
      } 
    else {if(EP_8 > 0)       
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8)/8;
      Trades = 8;
      }
    else {if(EP_7 > 0)       
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7)/7;
      Trades = 7;
      }
    else {if(EP_6 > 0)   
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6)/6;
      Trades = 6;
      }          
    else {if(EP_5 > 0)    
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5)/5;
      Trades = 5;
      }            
    else {if(EP_4 > 0)  
      {
      Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4)/4;
      Trades = 4;
      }
    else {if(EP_3 > 0)     
      {
      Avg_Price = (EP_1 + EP_2 + EP_3)/3;
      Trades = 3;
      }
    else {if(EP_2 > 0)     
      {
      Avg_Price = (EP_1 + EP_2)/2;
      Trades = 2;
      }    
    else {if(EP_1 > 0) 
      {
      Avg_Price= EP_1;
      Trades = 1;
      }                                     
      }}}}}}}}}}}}}}}}}}}               
    }
    
  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+  
int deinit()
  {
  obj_total= ObjectsTotal();  
  for(k= obj_total; k>=0; k--)
    {
    name= ObjectName(k);
    if (StringSubstr(name,0,7)=="[Trade]"){ObjectDelete(name);}
    }     
  //Comment("");    
  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+ 
int start()
  {
  //If indicator is "Off" or chart TF is out of range deinitialize only once, not every tick.  
  if((!Indicator_On) || ((Period() < Display_Min_TF) || (Period() > Display_Max_TF)))    
    {
    if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
    return(0);
    }

  //Otherwise indicator is "On" & chart TF is in display range, so proceed  
  deinit(); FLAG_deinit = false;  
            
  //Stop/start values for trade lines
  T1=Time[0]; //all lines (anchor point)  
 
  //T2= EP, SL, TP lines stop and labels start    
  //T3= avgEP & TP lines stop and labels start 
  //T4= selector for avgEP lines stop and labels start  
  //T5= selector for TP lines stop and labels start     
     
  //Stop/start values for trade lines for each chart zoom                   
  if(Chart_Zoom == 1) 
    {
    T2=Time[0]+(Period()*60*12);            
    T3=Time[0]+(Period()*60*108);   
    }     
  else if(Chart_Zoom == 2) 
    {
    T2=Time[0]+(Period()*60*6);   
    T3=Time[0]+(Period()*60*54);          
    }   
  else if(Chart_Zoom == 3) 
    {
    T2=Time[0]+(Period()*60*4);              
    T3=Time[0]+(Period()*60*28);     
    }
  else if(Chart_Zoom == 4) 
    {
    T2=Time[0]+(Period()*60*3);       
    T3=Time[0]+(Period()*60*15);    
    }
  else if(Chart_Zoom == 5) 
    {
    T2=Time[0]+(Period()*60*2);          
    T3=Time[0]+(Period()*60*8);  
    }
  else if(Chart_Zoom == 6) 
    {
    T2=Time[0]+(Period()*60*2);       
    T3=Time[0]+(Period()*60*5);    
    }
 
  //Right Shift Management
  if(Right_Shift_Avg_EP_and_TPs && EP_2>0)
    {
    //average EP & TP(s) shifted
    T4=T3;T5=T3;
    }
  else 
    {
    //average EP & TP(s) not shifted
    T4=T2;T5=T2;
    } 

  //Do Invidual EPs
  if(EP_1 != 0)
    {
    ObjectCreate(name01,OBJ_TREND,0,T2,EP_1,T1,EP_1);
    ObjectSet(name01,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name02,OBJ_TEXT,0,T2,EP_1); 
    EPstr = DoubleToStr(EP_1,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name02, spc+"EP 01:  " +EPstr, Label_Size, Label_Style, EP_Color);         
    }
  if(EP_2 != 0)
    {
    ObjectCreate(name03,OBJ_TREND,0,T2,EP_2,T1,EP_2);
    ObjectSet(name03,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name04,OBJ_TEXT,0,T2,EP_2);     
    EPstr = DoubleToStr(EP_2,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name04, spc+"EP 02:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }      
  if(EP_3 != 0)
    {
    ObjectCreate(name05,OBJ_TREND,0,T2,EP_3,T1,EP_3);
    ObjectSet(name05,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name06,OBJ_TEXT,0,T2,EP_3);     
    EPstr = DoubleToStr(EP_3,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name06, spc+"EP 03:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }
  if(EP_4 != 0)
    {
    ObjectCreate(name07,OBJ_TREND,0,T2,EP_4,T1,EP_4);
    ObjectSet(name07,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name08,OBJ_TEXT,0,T2,EP_4);     
    EPstr = DoubleToStr(EP_4,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name08, spc+"EP 04:  " +EPstr, Label_Size, Label_Style, EP_Color);      
    }      
  if(EP_5 != 0)
    {
    ObjectCreate(name09,OBJ_TREND,0,T2,EP_5,T1,EP_5);
    ObjectSet(name09,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name10,OBJ_TEXT,0,T2,EP_5);     
    EPstr = DoubleToStr(EP_5,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name10, spc+"EP 05:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }         
  if(EP_6 != 0)
    {
    ObjectCreate(name11,OBJ_TREND,0,T2,EP_6,T1,EP_6);
    ObjectSet(name11,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name12,OBJ_TEXT,0,T2,EP_6);     
    EPstr = DoubleToStr(EP_6,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name12, spc+"EP 06:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }
  if(EP_7 != 0)
    {
    ObjectCreate(name13,OBJ_TREND,0,T2,EP_7,T1,EP_7);
    ObjectSet(name13,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name14,OBJ_TEXT,0,T2,EP_7);     
    EPstr = DoubleToStr(EP_7,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name14, spc+"EP 07:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }      
  if(EP_8 != 0)
    {
    ObjectCreate(name15,OBJ_TREND,0,T2,EP_8,T1,EP_8);
    ObjectSet(name15,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name16,OBJ_TEXT,0,T2,EP_8);     
    EPstr = DoubleToStr(EP_8,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name16, spc+"EP 08:  " +EPstr, Label_Size, Label_Style, EP_Color);  
    }         
  if(EP_9 != 0)
    {
    ObjectCreate(name17,OBJ_TREND,0,T2,EP_9,T1,EP_9);
    ObjectSet(name17,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name18,OBJ_TEXT,0,T2,EP_9);     
    EPstr = DoubleToStr(EP_9,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name18, spc+"EP 09:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }        
  if(EP_10 != 0)
    {
    ObjectCreate(name19,OBJ_TREND,0,T2,EP_10,T1,EP_10);
    ObjectSet(name19,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name20,OBJ_TEXT,0,T2,EP_10);     
    EPstr = DoubleToStr(EP_10,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name20, spc+"EP 10:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }
  if(EP_11 != 0)
    {
    ObjectCreate(name21,OBJ_TREND,0,T2,EP_11,T1,EP_11);
    ObjectSet(name21,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name22,OBJ_TEXT,0,T2,EP_11);     
    EPstr = DoubleToStr(EP_11,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name22, spc+"EP 11:  " +EPstr, Label_Size, Label_Style, EP_Color);         
    }      
  if(EP_12 != 0)
    {
    ObjectCreate(name23,OBJ_TREND,0,T2,EP_12,T1,EP_12);
    ObjectSet(name23,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name24,OBJ_TEXT,0,T2,EP_12);     
    EPstr = DoubleToStr(EP_12,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name24, spc+"EP 12:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }
  if(EP_13 != 0)
    {
    ObjectCreate(name25,OBJ_TREND,0,T2,EP_13,T1,EP_13);
    ObjectSet(name25,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name26,OBJ_TEXT,0,T2,EP_13);     
    EPstr = DoubleToStr(EP_13,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name26, spc+"EP 13:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }      
  if(EP_14 != 0)
    {
    ObjectCreate(name27,OBJ_TREND,0,T2,EP_14,T1,EP_14);
    ObjectSet(name27,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name28,OBJ_TEXT,0,T2,EP_14);     
    EPstr = DoubleToStr(EP_14,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name28, spc+"EP 14:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }         
  if(EP_15 != 0)
    {
    ObjectCreate(name29,OBJ_TREND,0,T2,EP_15,T1,EP_15);
    ObjectSet(name29,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name30,OBJ_TEXT,0,T2,EP_15);     
    EPstr = DoubleToStr(EP_15,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name30, spc+"EP 15:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }                                                                     
  if(EP_16 != 0)
    {
    ObjectCreate(name31,OBJ_TREND,0,T2,EP_16,T1,EP_16);
    ObjectSet(name31,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name32,OBJ_TEXT,0,T2,EP_16);     
    EPstr = DoubleToStr(EP_16,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name32, spc+"EP 16:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }                                                                     
  if(EP_17 != 0)
    {
    ObjectCreate(name33,OBJ_TREND,0,T2,EP_17,T1,EP_17);
    ObjectSet(name33,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name34,OBJ_TEXT,0,T2,EP_17);     
    EPstr = DoubleToStr(EP_17,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name34, spc+"EP 17:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }                      
  if(EP_18 != 0)
    {
    ObjectCreate(name35,OBJ_TREND,0,T2,EP_18,T1,EP_18);
    ObjectSet(name35,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name36,OBJ_TEXT,0,T2,EP_18);     
    EPstr = DoubleToStr(EP_18,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name36, spc+"EP 18:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }
  if(EP_19 != 0)
    {
    ObjectCreate(name37,OBJ_TREND,0,T2,EP_19,T1,EP_19);
    ObjectSet(name37,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name38,OBJ_TEXT,0,T2,EP_19);     
    EPstr = DoubleToStr(EP_19,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name38, spc+"EP 19:  " +EPstr, Label_Size, Label_Style, EP_Color);  
    }
  if(EP_20 != 0)
    {
    ObjectCreate(name39,OBJ_TREND,0,T2,EP_20,T1,EP_20);
    ObjectSet(name39,OBJPROP_COLOR,EP_Color);
    ObjectCreate(name40,OBJ_TEXT,0,T2,EP_20);     
    EPstr = DoubleToStr(EP_20,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name40, spc+"EP 20:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }                                                                                                                                                                                                                                    
                  
  //Do Average of EPs 
  if(EP_2>0)
    {
    //Do Average EP
    ObjectCreate(name51,OBJ_TREND,0,T4,Avg_Price,T1,Avg_Price);
    ObjectSet(name51,OBJPROP_COLOR,EP_Av_Color);
    ObjectCreate(name52,OBJ_TEXT,0,T4,Avg_Price);
    EPstr = DoubleToStr(Avg_Price,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    if(Trades <10)
      {        
      ObjectSetText(name52,spc+"Av 0"+Trades+":  " +EPstr,Label_Size,Label_Style,EP_Av_Color);    
      }      
    else
      {  
      ObjectSetText(name52,spc+"Av "+Trades+":  " +EPstr,Label_Size,Label_Style,EP_Av_Color);
      }            
    }
    
  //Do Individual TPs    
  ObjectCreate(name61,OBJ_TREND,0,T5,TP_1,T1,TP_1);
  ObjectSet(name61,OBJPROP_COLOR,TP_Color);
  ObjectCreate(name62,OBJ_TEXT,0,T5,TP_1);
  EPstr = DoubleToStr(TP_1,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectSetText(name62, spc+"TP 01:  " +EPstr, Label_Size, Label_Style, TP_Color);          
        
  ObjectCreate(name63,OBJ_TREND,0,T5,TP_2,T1,TP_2);
  ObjectSet(name63,OBJPROP_COLOR,TP_Color); 
  ObjectCreate(name64,OBJ_TEXT,0,T5,TP_2);     
  EPstr = DoubleToStr(TP_2,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectSetText(name64, spc+"TP 02:  " +EPstr, Label_Size, Label_Style, TP_Color);          
          
  ObjectCreate(name65,OBJ_TREND,0,T5,TP_3,T1,TP_3);
  ObjectSet(name65,OBJPROP_COLOR,TP_Color);
  ObjectCreate(name66,OBJ_TEXT,0,T5,TP_3);     
  EPstr = DoubleToStr(TP_3,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectSetText(name66, spc+"TP 03:  " +EPstr, Label_Size, Label_Style, TP_Color);          
      
  //Do Individual SLs    
  ObjectCreate(name71,OBJ_TREND,0,T2,SL_1,T1,SL_1);
  ObjectSet(name71,OBJPROP_COLOR,SL_Color);    
  ObjectCreate(name72,OBJ_TEXT,0,T2,SL_1);
  EPstr = DoubleToStr(SL_1,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectSetText(name72, spc+"SL 01:  " +EPstr, Label_Size, Label_Style, SL_Color);          
         
  ObjectCreate(name73,OBJ_TREND,0,T2,SL_2,T1,SL_2);
  ObjectSet(name73,OBJPROP_COLOR,SL_Color);               
  ObjectCreate(name74,OBJ_TEXT,0,T2,SL_2);             
  EPstr = DoubleToStr(SL_2,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectSetText(name74, spc+"SL 02:  " +EPstr, Label_Size, Label_Style, SL_Color);          
   
  ObjectCreate(name75,OBJ_TREND,0,T2,SL_3,T1,SL_3);
  ObjectSet(name75,OBJPROP_COLOR,SL_Color);              
  ObjectCreate(name76,OBJ_TEXT,0,T2,SL_3);             
  EPstr = DoubleToStr(SL_3,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectSetText(name76, spc+"SL 03:  " +EPstr, Label_Size, Label_Style, SL_Color);          

          
  //Sound the alert for TPs and SLs
  if(Alert_on_Price && prevBid >0)
    {
    //Crossing __Bid_Price
    if((Bid >= __Bid_Price && prevBid < __Bid_Price) ||
      (Bid <= __Bid_Price && prevBid > __Bid_Price)) 
        {
        Alert(Broker_Name_In_Alert,":   ",Symbol(),"-",Period(),"   At Price ",
        NormalizeDouble(__Bid_Price,Digits));
        if(__Alert_Only_Once_Per_TF == true) {Alert_Allowed = false;}    
        }
     }
                    
  prevBid = Bid;
  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+   

